#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

from ...common_descs import *
from .objs.tag import HekTag
from supyr_struct.defs.tag_def import TagDef

shader = Struct("shader",
    Pad(36),
    FlUInt32("unknown0"),
    Bool16("shader_flags", *shader_flags),
    SEnum16("framebuffer_blend_function", *framebuffer_blend_functions),
    SEnum16("framebuffer_fade_mode", *render_fade_mode),
    Bool16("map_flags",
        "unfiltered"
        ),
    Pad(40),
    FlUInt32("unknown1"),
    Pad(88),
    SIZE=180
    )

marker = Struct("marker",
    ascii_str32("attachment_marker"),
    Bool16("flags",
        "not_connected_to_next_marker"
        ),

    Pad(2),
    SInt16("octaves_to_next_marker"),

    Pad(78),
    QStruct("random_position_bounds", INCLUDE=ijk_float, SIDETIP="world units"),
    float_wu("random_jitter"),
    float_wu("thickness"),
    QStruct("tint", INCLUDE=argb_float),
    SIZE=228
    )

elec_body = Struct("tagdata",
    Pad(2),
    SInt16("effects_count"),

    Pad(16),
    float_wu("near_fade_distance"),
    float_wu("far_fade_distance"),

    Pad(16),
    SEnum16("jitter_scale_source", *function_outputs),
    SEnum16("thickness_scale_source", *function_outputs),
    SEnum16("tint_modulation_source", *function_names),
    SEnum16("brightness_scale_source", *function_outputs),
    dependency("bitmap", "bitm"),

    Pad(84),
    reflexive("markers", marker, 16, DYN_NAME_PATH='.attachment_marker'),
    reflexive("shaders", shader, 1),

    SIZE=264,
    )


def get():
    return elec_def

elec_def = TagDef("elec",
    blam_header("elec"),
    elec_body,

    ext=".lightning", endian=">", tag_cls=HekTag,
    )



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

from ...common_descs import *
from .objs.wphi import WphiTag
from supyr_struct.defs.tag_def import TagDef
from .grhi import messaging_information, multitex_overlay, hud_background

crosshair_types = (
    "aim",
    "zoom",
    "charge",
    "should_reload",
    "flash_heat",
    "flash_total_ammo",
    "flash_battery",
    "reload_overheat",
    "flash_when_firing_and_no_ammo",
    "flash_when_throwing_grenade_and_no_grenade",
    "low_ammo_and_none_left_to_reload",
    "should_reload_secondary_trigger",
    "flash_secondary_total_ammo",
    "flash_secondary_reload",
    "flash_when_firing_secondary_and_no_ammo",
    "low_secondary_ammo_and_none_left_to_reload",
    "primary_trigger_ready",
    "secondary_trigger_ready",
    "flash_when_firing_with_depleted_battery",
    )

attached_state = SEnum16("state_attached_to",
    "total_ammo",
    "loaded_ammo",
    "heat",
    "age",
    "secondary_weapon_total_ammo",
    "secondary_weapon_loaded_ammo",
    "distance_to_target",
    "elevation_to_target",
    )

use_on_map_type = SEnum16("can_use_on_map_type",
    "any",
    "solo",
    "multiplayer",
    )

static_element = Struct("static_element",
    attached_state,
    Pad(2),
    use_on_map_type,

    Pad(30),
    QStruct("anchor_offset",
        SInt16("x"), SInt16("y"), ORIENT='h',
        ),
    Float("width_scale"),
    Float("height_scale"),
    Bool16("scaling_flags", *hud_scaling_flags),

    Pad(22),
    dependency("interface_bitmap", "bitm"),
    #QStruct("default_color", INCLUDE=argb_byte),
    #QStruct("flashing_color", INCLUDE=argb_byte),
    UInt32("default_color", INCLUDE=argb_uint32),
    UInt32("flashing_color", INCLUDE=argb_uint32),
    float_sec("flash_period"),
    float_sec("flash_delay"),
    SInt16("number_of_flashes"),
    Bool16("flash_flags", *hud_flash_flags),
    float_sec("flash_length"),
    #QStruct("disabled_color", INCLUDE=argb_byte),
    UInt32("disabled_color", INCLUDE=argb_uint32),

    Pad(4),
    SInt16("sequence_index"),

    Pad(2),
    reflexive("multitex_overlays", multitex_overlay, 30),
    SIZE=180
    )

meter_element = Struct("meter_element",
    attached_state,
    Pad(2),
    use_on_map_type,

    Pad(30),
    QStruct("anchor_offset",
        SInt16("x"), SInt16("y"), ORIENT='h',
        ),
    Float("width_scale"),
    Float("height_scale"),
    Bool16("scaling_flags", *hud_scaling_flags),

    Pad(22),
    dependency("meter_bitmap", "bitm"),
    #QStruct("color_at_meter_minimum", INCLUDE=xrgb_byte),
    #QStruct("color_at_meter_maximum", INCLUDE=xrgb_byte),
    #QStruct("flash_color", INCLUDE=xrgb_byte),
    #QStruct("empty_color", INCLUDE=argb_byte),
    UInt32("color_at_meter_minimum", INCLUDE=xrgb_uint32),
    UInt32("color_at_meter_maximum", INCLUDE=xrgb_uint32),
    UInt32("flash_color", INCLUDE=xrgb_uint32),
    UInt32("empty_color", INCLUDE=argb_uint32),
    Bool8("flags", *hud_panel_meter_flags),
    SInt8("minimum_meter_value"),
    SInt16("sequence_index"),
    SInt8("alpha_multiplier"),
    SInt8("alpha_bias"),
    SInt16("value_scale"),
    Float("opacity"),
    Float("translucency"),
    #QStruct("disabled_color", INCLUDE=argb_byte),
    UInt32("disabled_color", INCLUDE=argb_uint32),
    SIZE=180
    )

number_element = Struct("number_element",
    attached_state,
    Pad(2),
    use_on_map_type,

    Pad(30),
    QStruct("anchor_offset",
        SInt16("x"), SInt16("y"), ORIENT='h',
        ),
    Float("width_scale"),
    Float("height_scale"),
    Bool16("scaling_flags", *hud_scaling_flags),

    Pad(22),
    #QStruct("default_color", INCLUDE=argb_byte),
    #QStruct("flashing_color", INCLUDE=argb_byte),
    UInt32("default_color", INCLUDE=argb_uint32),
    UInt32("flashing_color", INCLUDE=argb_uint32),
    float_sec("flash_period"),
    float_sec("flash_delay"),
    SInt16("number_of_flashes"),
    Bool16("flash_flags", *hud_flash_flags),
    float_sec("flash_length"),
    #QStruct("disabled_color", INCLUDE=argb_byte),
    UInt32("disabled_color", INCLUDE=argb_uint32),

    Pad(4),
    SInt8("maximum_number_of_digits"),
    Bool8("flags",
        "show_leading_zeros",
        "only_show_when_zoomed",
        "draw_a_trailing_m",
        ),
    SInt8("number_of_fractional_digits"),
    Pad(1),

    Pad(12),
    Bool16("weapon_specific_flags",
        "divide_number_by_magazine_size"
        ),
    SIZE=160
    )

crosshair_overlay = Struct("crosshair_overlay",
    QStruct("anchor_offset",
        SInt16("x"), SInt16("y"), ORIENT='h',
        ),
    Float("width_scale"),
    Float("height_scale"),
    Bool16("scaling_flags", *hud_scaling_flags),

    Pad(22),
    #QStruct("default_color", INCLUDE=argb_byte),
    #QStruct("flashing_color", INCLUDE=argb_byte),
    UInt32("default_color", INCLUDE=argb_uint32),
    UInt32("flashing_color", INCLUDE=argb_uint32),
    float_sec("flash_period"),
    float_sec("flash_delay"),
    SInt16("number_of_flashes"),
    Bool16("flash_flags", *hud_flash_flags),
    float_sec("flash_length"),
    #QStruct("disabled_color", INCLUDE=argb_byte),
    UInt32("disabled_color", INCLUDE=argb_uint32),

    Pad(4),
    SInt16("frame_rate", UNIT_SCALE=per_sec_unit_scale),
    SInt16("sequence_index"),
    Bool32("type",
        "flashes_when_active",
        "not_a_sprite",
        "show_only_when_zoomed",
        "show_sniper_data",
        "hide_area_outside_reticle",
        "one_zoom_level",
        "dont_show_when_zoomed",
        ),
    SIZE=108
    )

overlay = Struct("overlay",
    QStruct("anchor_offset",
        SInt16("x"), SInt16("y"), ORIENT='h',
        ),
    Float("width_scale"),
    Float("height_scale"),
    Bool16("scaling_flags", *hud_scaling_flags),

    Pad(22),
    #QStruct("default_color", INCLUDE=argb_byte),
    #QStruct("flashing_color", INCLUDE=argb_byte),
    UInt32("default_color", INCLUDE=argb_uint32),
    UInt32("flashing_color", INCLUDE=argb_uint32),
    float_sec("flash_period"),
    float_sec("flash_delay"),
    SInt16("number_of_flashes"),
    Bool16("flash_flags", *hud_flash_flags),
    float_sec("flash_length"),
    #QStruct("disabled_color", INCLUDE=argb_byte),
    UInt32("disabled_color", INCLUDE=argb_uint32),

    Pad(4),
    SInt16("frame_rate", UNIT_SCALE=per_sec_unit_scale),
    Pad(2),
    SInt16("sequence_index"),
    Bool16("type",
        "show_on_flashing",
        "show_on_empty",
        "show_on_reload_overheating",
        "show_on_default",
        "show_always",
        ),
    Bool32("flags",
        "flashes_when_active",
        ),
    SIZE=136
    )

crosshair = Struct("crosshair",
    SEnum16("crosshair_type", *crosshair_types),
    Pad(2),
    use_on_map_type,

    Pad(30),
    dependency("crosshair_bitmap", "bitm"),
    reflexive("crosshair_overlays", crosshair_overlay, 16),
    SIZE=104
    )

overlay_element = Struct("overlay_element",
    attached_state,
    Pad(2),
    use_on_map_type,

    Pad(30),
    dependency("overlay_bitmap", "bitm"),
    reflexive("overlays", overlay, 16),
    SIZE=104
    )

screen_effect = Struct("screen_effect",
    Pad(4),
    Struct("mask",
        Bool16("flags",
            "only_when_zoomed"
            ),
        Pad(18),
        dependency("fullscreen_mask", "bitm"),
        dependency("splitscreen_mask", "bitm")
        ),

    Pad(8),
    Struct("convolution",
        Bool16("flags",
            "only_when_zoomed"
            ),
        Pad(2),
        from_to_rad("fov_in_bounds"),  # radians
        QStruct("radius_out_bounds",
            INCLUDE=from_to, SIDETIP="pixels")  # pixels
        ),

    Pad(24),
    Struct("night_vision",
        Bool16("flags",
            "only_when_zoomed",
            "connect_to_flashlight",
            "masked"
            ),
        SInt16("script_source", MIN=0, MAX=3, SIDETIP="[0,3]"),
        Float("intensity", MIN=0.0, MAX=1.0, SIDETIP="[0,1]")
        ),

    Pad(24),
    Struct("desaturation",
        Bool16("flags",
            "only_when_zoomed",
            "connect_to_flashlight",
            "additive",
            "masked"
            ),
        SInt16("script_source", MIN=0, MAX=3, SIDETIP="[0,3]"),
        Float("intensity", MIN=0.0, MAX=1.0, SIDETIP="[0,1]"),
        QStruct("tint", INCLUDE=rgb_float)
        ),
    SIZE=184
    )

wphi_body = Struct("tagdata",
    dependency("child_hud", "wphi"),
    Struct("flash_cutoffs",
        Bool16("flags",
            "use_parent_hud_flashing_parameters"
            ),
        Pad(2),
        SInt16("total_ammo_cutoff"),
        SInt16("loaded_ammo_cutoff"),
        SInt16("heat_cutoff"),
        SInt16("age_cutoff"),
        ),

    Pad(32),
    SEnum16("anchor", *hud_anchors),

    Pad(34),
    reflexive("static_elements", static_element, 16),
    reflexive("meter_elements", meter_element, 16),
    reflexive("number_elements", number_element, 16),
    reflexive("crosshairs", crosshair, 19),
    reflexive("overlay_elements", overlay_element, 16),
    FlBool32("crosshair_types", *crosshair_types, VISIBLE=False),

    # necessary for reticles to show up sometimes
    Pad(12),
    reflexive("screen_effect", screen_effect, 1),

    Pad(132),
    messaging_information,
    SIZE=380,
    )


def get():
    return wphi_def

wphi_def = TagDef("wphi",
    blam_header("wphi", 2),
    wphi_body,

    ext=".weapon_hud_interface", endian=">", tag_cls=WphiTag,
    )



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

from ...os_v3_hek.defs.bitm import *

def get(): return bitm_def

# replace the model animations dependency with an open sauce one
bitm_body = dict(bitm_body)
bitm_body[3] = Bool16("flags",
    "enable_diffusion_dithering",
    "disable_height_map_compression",
    "uniform_sprite_sequences",
    "sprite_bug_fix",
    ("never_share_resources", 1<<13)
    )

def get():
    return bitm_def

bitm_def = TagDef("bitm",
    blam_header('bitm', 7),
    bitm_body,

    ext=".bitmap", endian=">", tag_cls=BitmTag,
    subdefs = {'pixel_root':pixel_root}
    )



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

from ...common_descs import *
from .objs.snd_ import Snd_Tag
from supyr_struct.defs.tag_def import TagDef

sound_classes = (
    ("projectile_impact", 0),
    ("projectile_detonation", 1),

    ("weapon_fire", 4),
    ("weapon_ready", 5),
    ("weapon_reload", 6),
    ("weapon_empty", 7),
    ("weapon_charge", 8),
    ("weapon_overheat", 9),
    ("weapon_idle", 10),

    ("object_impacts", 13),
    ("particle_impacts", 14),
    ("slow_particle_impacts", 15),

    ("unit_footsteps", 18),
    ("unit_dialog", 19),

    ("vehicle_collision", 22),
    ("vehicle_engine", 23),

    ("device_door", 26),
    ("device_force_field", 27),
    ("device_machinery", 28),
    ("device_nature", 29),
    ("device_computers", 30),

    ("music", 32),
    ("ambient_nature", 33),
    ("ambient_machinery", 34),
    ("ambient_computers", 35),

    ("first_person_damage", 39),

    ("scripted_dialog_player", 44),
    ("scripted_effect", 45),
    ("scripted_dialog_other", 46),
    ("scripted_dialog_force_unspatialized", 47),

    ("game_event", 50),
    )

compression = SEnum16("compression",
    'none',
    'xbox_adpcm',
    'ima_adpcm',
    'ogg',
    TOOLTIP="""
IMA ADPCM is unsupported on pc, and is actually treated as
MS ADPCM, with an additional header on the data stream.
For all intents and purposes, "ima adpcm" is useless.
"""
    )


permutation = Struct('permutation',
    ascii_str32("name"),
    Float("skip_fraction"),
    Float("gain", DEFAULT=1.0),
    compression,
    SInt16("next_permutation_index", DEFAULT=-1),
    FlSInt32("unknown0", VISIBLE=False),
    FlUInt32("unknown1", VISIBLE=False),  # always zero?
    FlUInt32("unknown2", VISIBLE=False),
    # this is actually the required length of the ogg
    # decompression buffer. For "none" compression, this
    # mirrors samples.size, so a more appropriate name
    # for this field should be pcm_buffer_size
    FlUInt32("ogg_sample_count", EDITABLE=False),
    FlUInt32("unknown3", VISIBLE=False),  # seems to always be == unknown2
    rawdata_ref("samples", max_size=4194304, widget=SoundSampleFrame),
    rawdata_ref("mouth_data", max_size=8192),
    rawdata_ref("subtitle_data", max_size=512),
    SIZE=124
    )

pitch_range = Struct('pitch_range',
    ascii_str32("name"),

    Float("natural_pitch"),
    QStruct("bend_bounds", INCLUDE=from_to),
    SInt16("actual_permutation_count"),
    Pad(2),
    Float("playback_rate", VISIBLE=False),
    SInt32("unknown1", VISIBLE=False, DEFAULT=-1),
    SInt32("unknown2", VISIBLE=False, DEFAULT=-1),

    reflexive("permutations", permutation, 256,
        DYN_NAME_PATH='.name', IGNORE_SAFE_MODE=True),
    SIZE=72,
    )


snd__body = Struct("tagdata",
    Bool32("flags",
        "fit_to_adpcm_blocksize",
        "split_long_sound_into_permutations"
        ),
    SEnum16("sound_class", *sound_classes),
    SEnum16("sample_rate",
        {NAME: "khz_22", GUI_NAME: "22kHz"},
        {NAME: "khz_44", GUI_NAME: "44kHz"},
        ),
    float_wu("minimum_distance"),
    float_wu("maximum_distance"),
    float_zero_to_one("skip_fraction"),

    #Randomization
    QStruct("random_pitch_bounds", INCLUDE=from_to),
    float_rad("inner_cone_angle"),  # radians
    float_rad("outer_cone_angle"),  # radians
    float_zero_to_one("outer_cone_gain"),
    Float("gain_modifier"),
    Float("maximum_bend_per_second"),
    Pad(12),

    QStruct("modifiers_when_scale_is_zero",
        Float("skip_fraction"),
        Float("gain"),
        Float("pitch"),
        ),
    Pad(12),

    QStruct("modifiers_when_scale_is_one",
        Float("skip_fraction"),
        Float("gain"),
        Float("pitch"),
        ),
    Pad(12),

    SEnum16("encoding",
        'mono',
        'stereo'
        ),
    compression,
    dependency("promotion_sound", "snd!"),
    SInt16("promotion_count"),
    SInt16("unknown1", VISIBLE=False),
    Struct("unknown2", INCLUDE=rawdata_ref_struct, VISIBLE=False),
    reflexive("pitch_ranges", pitch_range, 8,
        DYN_NAME_PATH='.name'),

    SIZE=164,
    )


def get():
    return snd__def

snd__def = TagDef("snd!",
    blam_header('snd!', 4),
    snd__body,

    ext=".sound", endian=">", tag_cls=Snd_Tag,
    )


snd__meta_stub = dict(snd__body)
snd__meta_stub[23] = Pad(12)
snd__meta_stub_blockdef = BlockDef(snd__meta_stub)



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

from ...hek.defs.obje import *
from ..common_descs import *

def get():
    return obje_def

obje_attrs = dict(obje_attrs)
obje_attrs[7] = dependency_stubbs('model', 'mode')

obje_body = Struct('tagdata',
    obje_attrs,
    SIZE=380
    )

obje_def = TagDef("obje",
    blam_header_stubbs('obje'),
    obje_body,

    ext=".object", endian=">", tag_cls=ObjeTag
    )

def get():
    return obje_def



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

############# Credits and version info #############
# Definition generated from Assembly XML tag def
#	 Date generated: 2018/12/03  04:56
#
# revision: 1		author: Assembly
# 	Generated plugin from scratch.
# revision: 2		author: DeadCanadian
# 	named a few things and set the types of a few
# revision: 3		author: Lord Zedd
# 	I don't always add revisions, but when I do it's because I spend hours making these giant enums.
# revision: 4		author: Moses_of_Egypt
# 	Cleaned up and converted to SuPyr definition
#
####################################################

from ..common_descs import *
from .objs.tag import *
from supyr_struct.defs.tag_def import TagDef

goof_game_engine_setting_setting_category = (
    "none",
    "global_respawn_options",
    "global_social_options",
    "global_map_overrides",
    "global_base_traits_main",
    "global_base_traits_health",
    "global_base_traits_weapons",
    "global_base_traits_movement",
    "global_base_traits_sensors",
    "global_base_traits_appearance",
    "unknown_0",
    "global_respawn_traits_main",
    "global_respawn_traits_health",
    "global_respawn_traits_weapons",
    "global_respawn_traits_movement",
    "global_respawn_traits_sensors",
    "global_respawn_traits_appearance",
    "unknown_1",
    "global_powerups_red_traits_main",
    "global_powerups_red_traits_health",
    "global_powerups_red_traits_weapons",
    "global_powerups_red_traits_movement",
    "global_powerups_red_traits_sensors",
    "global_powerups_red_traits_appearance",
    "unknown_2",
    "global_powerups_blue_traits_main",
    "global_powerups_blue_traits_health",
    "global_powerups_blue_traits_weapons",
    "global_powerups_blue_traits_movement",
    "global_powerups_blue_traits_sensors",
    "global_powerups_blue_traits_appearance",
    "unknown_3",
    "global_powerups_yellow_traits_main",
    "global_powerups_yellow_traits_health",
    "global_powerups_yellow_traits_weapons",
    "global_powerups_yellow_traits_movement",
    "global_powerups_yellow_traits_sensors",
    "global_powerups_yellow_traits_appearance",
    "unknown_4",
    "unknown_5",
    "slayer_leader_traits_main",
    "slayer_leader_traits_health",
    "slayer_leader_traits_weapons",
    "slayer_leader_traits_movement",
    "slayer_leader_traits_sensors",
    "slayer_leader_traits_appearance",
    "unknown_6",
    "oddball_carrier_traits_main",
    "oddball_carrier_traits_health",
    "oddball_carrier_traits_weapons",
    "oddball_carrier_traits_movement",
    "oddball_carrier_traits_sensors",
    "oddball_carrier_traits_appearance",
    "unknown_7",
    "assault_carrier_traits_main",
    "assault_carrier_traits_health",
    "assault_carrier_traits_weapons",
    "assault_carrier_traits_movement",
    "assault_carrier_traits_sensors",
    "assault_carrier_traits_appearance",
    "unknown_8",
    "ctf_carrier_traits_main",
    "ctf_carrier_traits_health",
    "ctf_carrier_traits_weapons",
    "ctf_carrier_traits_movement",
    "ctf_carrier_traits_sensors",
    "ctf_carrier_traits_appearance",
    "unknown_9",
    "juggernaut_juggy_traits_main",
    "juggernaut_juggy_traits_health",
    "juggernaut_juggy_traits_weapons",
    "juggernaut_juggy_traits_movement",
    "juggernaut_juggy_traits_sensors",
    "juggernaut_juggy_traits_appearance",
    "unknown_10",
    "koth_hill_traits_main",
    "koth_hill_traits_health",
    "koth_hill_traits_weapons",
    "koth_hill_traits_movement",
    "koth_hill_traits_sensors",
    "koth_hill_traits_appearance",
    "unknown_11",
    "infection_zombie_traits_main",
    "infection_zombie_traits_health",
    "infection_zombie_traits_weapons",
    "infection_zombie_traits_movement",
    "infection_zombie_traits_sensors",
    "infection_zombie_traits_appearance",
    "unknown_12",
    "infection_alpha_zombie_traits_main",
    "infection_alpha_zombie_traits_health",
    "infection_alpha_zombie_traits_weapons",
    "infection_alpha_zombie_traits_movement",
    "infection_alpha_zombie_traits_sensors",
    "infection_alpha_zombie_traits_appearance",
    "unknown_13",
    "infection_safe_haven_traits_main",
    "infection_safe_haven_traits_health",
    "infection_safe_haven_traits_weapons",
    "infection_safe_haven_traits_movement",
    "infection_safe_haven_traits_sensors",
    "infection_safe_haven_traits_appearance",
    "unknown_14",
    "infection_last_man_traits_main",
    "infection_last_man_traits_health",
    "infection_last_man_traits_weapons",
    "infection_last_man_traits_movement",
    "infection_last_man_traits_sensors",
    "infection_last_man_traits_appearance",
    "unknown_15",
    "unknown_unknown_traits_main",
    "unknown_unknown_traits_health",
    "unknown_unknown_traits_weapons",
    "unknown_unknown_traits_movement",
    "unknown_unknown_traits_sensors",
    "unknown_unknown_traits_appearance",
    "unknown_16",
    "territories_defender_traits_main",
    "territories_defender_traits_health",
    "territories_defender_traits_weapons",
    "territories_defender_traits_movement",
    "territories_defender_traits_sensors",
    "territories_defender_traits_appearance",
    "unknown_17",
    "territories_attacker_traits_main",
    "territories_attacker_traits_health",
    "territories_attacker_traits_weapons",
    "territories_attacker_traits_movement",
    "territories_attacker_traits_sensors",
    "territories_attacker_traits_appearance",
    "unknown_18",
    "vip_team_traits_main",
    "vip_team_traits_health",
    "vip_team_traits_weapons",
    "vip_team_traits_movement",
    "vip_team_traits_sensors",
    "vip_team_traits_appearance",
    "unknown_19",
    "vip_influence_traits_main",
    "vip_influence_traits_health",
    "vip_influence_traits_weapons",
    "vip_influence_traits_movement",
    "vip_influence_traits_sensors",
    "vip_influence_traits_appearance",
    "unknown_20",
    "vip_vip_traits_main",
    "vip_vip_traits_health",
    "vip_vip_traits_weapons",
    "vip_vip_traits_movement",
    "vip_vip_traits_sensors",
    "vip_vip_traits_appearance",
    "unknown_21",
    "editor_editor_traits_main",
    "editor_editor_traits_health",
    "editor_editor_traits_weapons",
    "editor_editor_traits_movement",
    "editor_editor_traits_sensors",
    "editor_editor_traits_appearance",
    "unknown_22",
    "ctf_top",
    "slayer_top",
    "oddball_top",
    "koth_top",
    "editor_top",
    "juggernaut_top",
    "territories_top",
    "assault_top",
    "infection_top",
    "vip_top",
    "unknown_23",
    "global_powerups",
    "global_respawn_advanced",
    "global_respawn_modifiers",
    "slayer_main",
    "unknown_24",
    "slayer_scoring",
    "oddball_main",
    "oddball_scoring",
    "assault_main",
    "assault_scoring",
    "ctf_main",
    "ctf_scoring",
    "juggernaut_main",
    "juggernaut_scoring",
    "koth_main",
    "koth_scoring",
    "infection_main",
    "infection_advanced",
    "infection_scoring",
    "unknown_25",
    "unknown_26",
    "unknown_27",
    "unknown_28",
    "unknown_29",
    "unknown_30",
    "unknown_31",
    "unknown_32",
    "unknown_33",
    "unknown_34",
    "unknown_35",
    "unknown_36",
    "vip_main",
    "vip_scoring",
    "territories_main",
    "territories_scoring",
    "templates_traits_main",
    "templates_traits_health",
    "templates_traits_weapons",
    "templates_traits_movement",
    "templates_traits_sensors",
    "templates_traits_appearance",
    "templates_dynamic_traits_main",
    "templates_dynamic_traits_health",
    "templates_dynamic_traits_weapons",
    "templates_dynamic_traits_movement",
    "templates_dynamic_traits_sensors",
    "templates_dynamic_traits_appearance",
    )


goof_game_engine_setting_option = Struct("option",
    h3_dependency("explicit_submenu"),
    h3_dependency("template_based_submenu"),
    SEnum32("submenu_setting_category", *goof_game_engine_setting_setting_category),
    h3_string_id("submenu_name"),
    h3_string_id("submenu_description"),
    h3_dependency("value_pairs"),
    ENDIAN=">", SIZE=60
    )


goof_game_engine_setting = Struct("game_engine_setting",
    h3_string_id("name"),
    SEnum32("setting_category", *goof_game_engine_setting_setting_category),
    h3_reflexive("options", goof_game_engine_setting_option),
    ENDIAN=">", SIZE=20
    )


goof_body = Struct("tagdata",
    SInt32("unknown", VISIBLE=False),
    h3_reflexive("game_engine_settings", goof_game_engine_setting),
    ENDIAN=">", SIZE=16
    )


def get():
    return goof_def

goof_def = TagDef("goof",
    h3_blam_header('goof'),
    goof_body,

    ext=".%s" % h3_tag_class_fcc_to_ext["goof"], endian=">", tag_cls=H3Tag
    )



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

from ..common_descs import *
from supyr_struct.defs.tag_def import TagDef

def get():
    return m2ed_def

m2ed_def = TagDef("m2ed",
    blam_header('m2ed'),
    QStruct('tagdata',
        ),

    ext=".magic_2_effect_data", endian=">"
    )



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

############# Credits and version info #############
# Definition generated from Assembly XML tag def
#	 Date generated: 2018/12/03  04:56
#
# revision: 1		author: -DeToX-
# 	Mapped plugin structure a new.
# revision: 2		author: DeadCanadian
# 	named most
# revision: 3		author: Lord Zedd
# 	h22222
# revision: 4		author: Moses_of_Egypt
# 	Cleaned up and converted to SuPyr definition
#
####################################################

from ..common_descs import *
from .objs.tag import *
from supyr_struct.defs.tag_def import TagDef

foot_effect_old_material_sweetener_mode = (
    "sweetener_default",
    "sweetener_enabled",
    "sweetener_disabled",
    )


foot_effect_old_material = Struct("old_material",
    h3_dependency("effect"),
    h3_dependency("sound"),
    h3_string_id("material_name"),
    SInt16("global_material_index"),
    SEnum8("sweetener_mode", *foot_effect_old_material_sweetener_mode),
    SInt8("unknown", VISIBLE=False),
    ENDIAN=">", SIZE=40
    )


foot_effect_sound = Struct("sound",
    h3_dependency("tag"),
    h3_dependency("secondary_tag"),
    h3_string_id("material_name"),
    SInt16("global_material_index"),
    SEnum8("sweetener_mode", *foot_effect_old_material_sweetener_mode),
    SInt8("unknown", VISIBLE=False),
    ENDIAN=">", SIZE=40
    )


foot_effect_effect = Struct("effect",
    h3_dependency("tag"),
    h3_dependency("secondary_tag"),
    h3_string_id("material_name"),
    SInt16("global_material_index"),
    SEnum8("sweetener_mode", *foot_effect_old_material_sweetener_mode),
    SInt8("unknown", VISIBLE=False),
    ENDIAN=">", SIZE=40
    )


foot_effect = Struct("effect",
    h3_reflexive("old_materials", foot_effect_old_material),
    h3_reflexive("sounds", foot_effect_sound),
    h3_reflexive("effects", foot_effect_effect),
    ENDIAN=">", SIZE=36
    )


foot_body = Struct("tagdata",
    h3_reflexive("effects", foot_effect),
    ENDIAN=">", SIZE=12
    )


def get():
    return foot_def

foot_def = TagDef("foot",
    h3_blam_header('foot'),
    foot_body,

    ext=".%s" % h3_tag_class_fcc_to_ext["foot"], endian=">", tag_cls=H3Tag
    )



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

from ...common_descs import *
from .objs.tag import HekTag
from supyr_struct.defs.tag_def import TagDef

particle_creation_physics = (
    "default",
    "explosion",
    "jet"
    )

physics_constant = QStruct("physics_constant",
    Float("k", UNIT_SCALE=per_sec_unit_scale),
    )

state = Struct("state",
    ascii_str32("name"),
    from_to_sec("duration_bounds"),
    from_to_sec("transition_time_bounds"),

    Pad(4),
    Float("scale_multiplier"),
    Float("animation_rate_multiplier"),
    Float("rotation_rate_multiplier"),
    QStruct("color_multiplier", INCLUDE=argb_float),
    Float("radius_multiplier"),
    Float("minimum_particle_count"),
    Float("particle_creation_rate",
           SIDETIP="particles/sec", UNIT_SCALE=per_sec_unit_scale),

    Pad(84),
    SEnum16("particle_creation_physics", *particle_creation_physics),
    Pad(2),  # SEnum16("particle_update_physics", "default"),
    reflexive("physics_constants", physics_constant, 16),
    SIZE=192
    )

particle_state = Struct("particle_state",
    ascii_str32("name"),
    from_to_sec("duration_bounds"),
    from_to_sec("transition_time_bounds"),

    dependency("bitmaps", "bitm"),
    SInt16("sequence_index"),
    Pad(6),
    QStruct("scale", INCLUDE=from_to, SIDETIP="world units/pixel"),
    QStruct("animation_rate",
        Float("from", UNIT_SCALE=per_sec_unit_scale, GUI_NAME=''),
        Float("to",   UNIT_SCALE=per_sec_unit_scale),
        ORIENT='h', SIDETIP="frames/sec"
        ),
    from_to_rad_sec("rotation_rate"),  # radians/sec
    QStruct("color_1", INCLUDE=argb_float),
    QStruct("color_2", INCLUDE=argb_float),
    Float("radius_multiplier"),
    dependency("physics", "pphy"),

    Pad(72),
    FlUInt32("unknown0", VISIBLE=False),
    Bool16("shader_flags", *shader_flags),
    SEnum16("framebuffer_blend_function", *framebuffer_blend_functions),
    SEnum16("framebuffer_fade_mode", *render_fade_mode),
    Bool16("map_flags",
        "unfiltered"
        ),

    Pad(12),  # open sauce particle shader extension
    Pad(16),
    #Secondary map
    Struct("secondary_map",
        dependency("bitmap", "bitm"),
        SEnum16("anchor", *render_anchor),
        Bool16("flags",
            "unfiltered"
            ),
        Struct("u_animation", INCLUDE=anim_src_func_per_pha_sca),
        Struct("v_animation", INCLUDE=anim_src_func_per_pha_sca),
        Struct("rotation_animation", INCLUDE=anim_src_func_per_pha_sca_rot),
        QStruct("rotation_center", INCLUDE=xy_float),
        Pad(4),
        Float("zsprite_radius_scale"),
        ),

    Pad(20),
    reflexive("physics_constants", physics_constant, 16),
    SIZE=376
    )


particle_type = Struct("particle_type",
    ascii_str32("name"),
    Bool32("flags",
        "type_states_loop",
        "type_states_loops_forward_backward",
        "particle_states_loop",
        "particle_states_loops_forward_backward",
        "particles_die_in_water",
        "particles_die_in_air",
        "particles_die_on_ground",
        "rotational_sprites_animate_sideways",
        "disabled",
        "tint_by_effect_color",
        "initial_count_scales_with_effect",
        "minimum_count_scales_with_effect",
        "creation_rate_scales_with_effect",
        "radius_scales_with_effect",
        "animation_rate_scales_with_effect",
        "rotation_rate_scales_with_effect",
        "dont_draw_in_first_person",
        "dont_draw_in_third_person",
        ),
    SInt16("initial_particle_count"),

    Pad(2),
    SEnum16("complex_sprite_render_modes",
        "simple",
        "rotational"
        ),

    Pad(2),
    float_wu("radius"),

    Pad(36),
    SEnum16("particle_creation_physics", *particle_creation_physics),

    Pad(6),
    reflexive("physics_constants", physics_constant, 16),
    reflexive("states", state, 8, DYN_NAME_PATH='.name'),
    reflexive("particle_states", particle_state, 8, DYN_NAME_PATH='.name'),
    SIZE=128
    )

pctl_body = Struct("tagdata",
    Pad(56),
    dependency("point_physics", "pphy"),
    SEnum16("system_update_physics",
        "default",
        "explosion"
        ),
    Pad(6),
    reflexive("physics_constants", physics_constant, 16),
    reflexive("particle_types", particle_type, 4, DYN_NAME_PATH='.name'),
    SIZE=104,
    )


def get():
    return pctl_def

pctl_def = TagDef("pctl",
    blam_header("pctl", 4),
    pctl_body,

    ext=".particle_system", endian=">", tag_cls=HekTag,
    )



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

############# Credits and version info #############
# Definition generated from Assembly XML tag def
#	 Date generated: 2018/12/03  04:56
#
# revision: 1		author: Assembly
# 	Generated plugin from scratch.
# revision: 2		author: -DeToX-
# 	Named Some Values...
# revision: 3		author: Halodu03de
# 	added chunk code
# revision: 4		author: Lord Zedd
# 	Lots of updates
# revision: 5		author: Moses_of_Egypt
# 	Cleaned up and converted to SuPyr definition
#
####################################################

from ..common_descs import *
from .objs.tag import *
from supyr_struct.defs.tag_def import TagDef

hlmt_model_object_data_type = (
    "not_set",
    "user_defined",
    "auto_generated",
    )

hlmt_new_damage_info_damage_section_instant_response_constraint_damage_type = (
    "none",
    "destroy_one_of_group",
    "destroy_entire_group",
    "loosen_one_of_group",
    "loosen_entire_group",
    )

hlmt_new_damage_info_damage_section_instant_response_response_type = (
    "recieves_all_damage",
    "recieves_area_effect_damage",
    "recieves_local_damage",
    )

hlmt_new_damage_info_damage_section_instant_response_unknown_special_damage = (
    "none",
    "_1",
    "_2",
    "_3",
    )

hlmt_variant_region_sort_order = (
    "no_sorting",
    "neg_5_closest",
    "neg_4",
    "neg_3",
    "neg_2",
    "neg_1",
    "_0_same_as_model",
    "_1",
    "_2",
    "_3",
    "_4",
    "_5_farthest",
    )


hlmt_variant_region_permutation_state = Struct("state",
    h3_string_id("name"),
    SInt8("model_permutation_index"),
    Bool8("property_flags",
        "blurred",
        "hella_blurred",
        "shielded",
        ),
    SEnum16("state", *vehi_friction_point_model_state_destroyed),
    h3_dependency("looping_effect"),
    h3_string_id("looping_effect_marker_name"),
    Float("initial_probability"),
    ENDIAN=">", SIZE=32
    )


hlmt_variant_region_permutation = Struct("permutation",
    h3_string_id("name"),
    SInt8("model_permutation_index"),
    Bool8("flags",
        "copy_states_to_all_permutations",
        ),
    SInt8("unknown_0", VISIBLE=False),
    SInt8("unknown_1", VISIBLE=False),
    Float("probability"),
    h3_reflexive("states", hlmt_variant_region_permutation_state),
    BytesRaw("unknown_2", SIZE=12, VISIBLE=False),
    ENDIAN=">", SIZE=36
    )


hlmt_variant_region = Struct("region",
    h3_string_id("name"),
    SInt8("model_region_index"),
    SInt8("unknown", VISIBLE=False),
    SInt16("parent_variant_index"),
    h3_reflexive("permutations", hlmt_variant_region_permutation),
    SEnum32("sort_order", *hlmt_variant_region_sort_order),
    ENDIAN=">", SIZE=24
    )


hlmt_variant_object = Struct("object",
    h3_string_id("parent_marker"),
    h3_string_id("child_marker"),
    h3_string_id("child_variant"),
    h3_dependency("child_object"),
    ENDIAN=">", SIZE=28
    )


hlmt_variant = Struct("variant",
    h3_string_id("name"),
    Array("model_region_index_array", SUB_STRUCT=SInt8("model_region_index"), SIZE=16),
    h3_reflexive("regions", hlmt_variant_region),
    h3_reflexive("objects", hlmt_variant_object),
    SInt32("instance_group_index"),
    BytesRaw("unknown", SIZE=8, VISIBLE=False),
    ENDIAN=">", SIZE=56
    )


hlmt_instance_group_instance_member = Struct("instance_member",
    SInt32("unknown", VISIBLE=False),
    h3_string_id("instance_name"),
    Float("probability"),
    Bool32("instance_flags_1", *("instance_%s" % i for i in range(32))),
    Bool32("instance_flags_2", *("instance_%s" % i for i in range(32, 64))),
    ENDIAN=">", SIZE=20
    )


hlmt_instance_group = Struct("instance_group",
    h3_string_id("name"),
    SInt32("unknown", VISIBLE=False),
    h3_reflexive("instance_members", hlmt_instance_group_instance_member),
    Float("probability"),
    ENDIAN=">", SIZE=24
    )


hlmt_material = Struct("material",
    h3_string_id("name"),
    SInt16("unknown_0", VISIBLE=False),
    SInt16("damage_section_index"),
    SInt16("unknown_1", VISIBLE=False),
    SInt16("unknown_2", VISIBLE=False),
    h3_string_id("material_name"),
    SInt16("global_material_index"),
    SInt16("unknown_3", VISIBLE=False),
    ENDIAN=">", SIZE=20
    )


hlmt_new_damage_info_damage_section_instant_response = Struct("instant_response",
    SEnum16("response_type", *hlmt_new_damage_info_damage_section_instant_response_response_type),
    SEnum16("constraint_damage_type", *hlmt_new_damage_info_damage_section_instant_response_constraint_damage_type),
    h3_string_id("trigger"),
    Bool32("flags",
        "kills_object",
        "inhibits_melee_attack",
        "inhibits_weapon_attack",
        "inhibits_walking",
        "forces_drop_weapon",
        "kills_weapon_primary_trigger",
        "kills_weapon_secondary_trigger",
        "destroys_object",
        "damages_weapon_primary_trigger",
        "damages_weapon_secondary_trigger",
        "light_damage_left_turn",
        "major_damage_left_turn",
        "light_damage_right_turn",
        "major_damage_right_turn",
        "light_damage_engine",
        "major_damage_engine",
        "kills_object_no_player_solo",
        "causes_detonation",
        "destroy_all_group_constraints",
        "kills_variant_objects",
        "force_unattached_effects",
        "fires_under_threshold",
        "triggers_special_death",
        "only_on_special_death",
        "only_not_on_special_death",
        ("causes_detonation_in_single_player", 1 << 26),
        ),
    Float("damage_threshold"),
    h3_dependency("primary_transition_effect"),
    h3_dependency("secondary_transition_effect"),
    h3_dependency("transition_damage_effect"),
    h3_string_id("region"),
    SEnum16("new_state", *vehi_friction_point_model_state_destroyed),
    SInt16("runtime_region_index"),
    h3_string_id("secondary_region"),
    SEnum16("secondary_new_state", *vehi_friction_point_model_state_destroyed),
    SInt16("secondary_runtime_region_index"),
    SInt16("unknown"),
    SEnum16("unknown_special_damage", *hlmt_new_damage_info_damage_section_instant_response_unknown_special_damage),
    h3_string_id("special_damage_case"),
    h3_string_id("effect_marker_name"),
    h3_string_id("damage_effect_marker_name"),
    Float("response_delay"),
    h3_dependency("delay_effect"),
    h3_string_id("delay_effect_marker_name"),
    h3_string_id("ejecting_seat_label"),
    Float("skip_fraction"),
    h3_string_id("destroyed_child_object_marker_name"),
    Float("total_damage_threshold"),
    ENDIAN=">", SIZE=136
    )


hlmt_new_damage_info_damage_section = Struct("damage_section",
    h3_string_id("name"),
    Bool32("flags",
        "absorbs_body_damage",
        "takes_full_damage_when_object_dies",
        "cannot_die_with_riders",
        "takes_full_damage_when_object_destroyed",
        "restored_on_ressurection",
        ("headshotable", 1 << 7),
        "ignores_shields",
        ),
    Float("vitality_percentage"),
    h3_reflexive("instant_responses", hlmt_new_damage_info_damage_section_instant_response),
    BytesRaw("unknown_0", SIZE=24, VISIBLE=False),
    Float("stun_time"),
    Float("recharge_time"),
    Float("unknown_1", VISIBLE=False),
    h3_string_id("resurrection_region_name"),
    SInt16("ressurection_region_runtime_index"),
    SInt16("unknown_2", VISIBLE=False),
    ENDIAN=">", SIZE=68
    )


hlmt_new_damage_info_node = Struct("node",
    SInt16("unknown_0", VISIBLE=False),
    SInt16("unknown_1", VISIBLE=False),
    BytesRaw("unknown_2", SIZE=12, VISIBLE=False),
    VISIBLE=False,
    ENDIAN=">", SIZE=16
    )


hlmt_new_damage_info_damage_seat_unknown = Struct("unknown",
    h3_string_id("node"),
    SInt16("unknown_0", VISIBLE=False),
    SInt16("unknown_1", VISIBLE=False),
    BytesRaw("unknown_2", SIZE=36, VISIBLE=False),
    VISIBLE=False,
    ENDIAN=">", SIZE=44
    )


hlmt_new_damage_info_damage_seat = Struct("damage_seat",
    h3_string_id("seat_label"),
    Float("direct_damage_scale"),
    Float("damage_transfer_fall_off_radius"),
    Float("maximum_transfer_damage_scale"),
    Float("minimum_transfer_damage_scale"),
    h3_reflexive("unknown", hlmt_new_damage_info_damage_seat_unknown),
    ENDIAN=">", SIZE=32
    )


hlmt_new_damage_info_damage_constraint = Struct("damage_constraint",
    h3_string_id("physics_model_constraint_name"),
    h3_string_id("damage_constraint_name"),
    h3_string_id("damage_constraint_group_name"),
    Float("group_probability_scale"),
    SEnum16("type", *phmo_node_edge_constraint_type),
    SInt16("idx"),
    ENDIAN=">", SIZE=20
    )


hlmt_new_damage_info = Struct("new_damage_info",
    Bool32("flags",
        "takes_shield_damage_for_children",
        "takes_body_damage_for_children",
        "always_shields_friendly_damage",
        "passes_area_damage_to_children",
        "parent_never_takes_body_damage_for_children",
        "only_damaged_by_explosives",
        "parent_never_takes_shield_damage_for_children",
        "cannot_die_from_damage",
        "passes_attached_damage_to_riders",
        ("only_damaged_by_player", 1 << 11),
        ),
    h3_string_id("global_indirect_material_name"),
    SInt16("indirect_damage_section"),
    SInt16("unknown_0", VISIBLE=False),
    BytesRaw("unknown_1", SIZE=4, VISIBLE=False),
    SEnum8("collision_damage_reporting_type", *proj_damage_reporting_type),
    SEnum8("response_damage_reporting_type", *proj_damage_reporting_type),
    SInt16("unknown_2", VISIBLE=False),
    BytesRaw("unknown_3", SIZE=20, VISIBLE=False),
    Float("maximum_vitality"),
    Float("minimum_stun_damage_0"),
    Float("stun_time_0"),
    Float("recharge_time"),
    Float("recharge_fraction"),
    Pad(64),
    Float("maximum_shield_vitality"),
    h3_string_id("global_shield_material_name"),
    Float("minimum_stun_damage_1"),
    Float("stun_time_1"),
    Float("shield_recharge_time"),
    Float("shield_damaged_threshold"),
    h3_dependency("shield_damaged_effect"),
    h3_dependency("shield_depleted_effect"),
    h3_dependency("shield_recharging_effect"),
    h3_reflexive("damage_sections", hlmt_new_damage_info_damage_section),
    h3_reflexive("nodes", hlmt_new_damage_info_node),
    SInt16("global_shield_material_index"),
    SInt16("global_indirect_material_index"),
    Float("unknown_5", VISIBLE=False),
    Float("unknown_6", VISIBLE=False),
    h3_reflexive("damage_seats", hlmt_new_damage_info_damage_seat),
    h3_reflexive("damage_constraints", hlmt_new_damage_info_damage_constraint),
    ENDIAN=">", SIZE=256
    )


hlmt_target = Struct("target",
    h3_string_id("marker_name"),
    Float("size"),
    float_rad("cone_angle", VISIBLE=False),
    SInt16("damage_section"),
    SInt16("variant"),
    Float("targeting_relevance"),
    BytesRaw("unknown", SIZE=4, VISIBLE=False),
    Bool32("flags",
        "locked_by_human_tracking",
        "locked_by_plasma_tracking",
        "headshot",
        ("vulnerable", 1 << 4),
        ("always_locked_by_plasma_tracking", 1 << 6),
        ),
    Float("lock_on_distance"),
    VISIBLE=False,
    ENDIAN=">", SIZE=32
    )


hlmt_collision_region_permutation = Struct("permutation",
    h3_string_id("name"),
    Bool8("flags",
        "cannot_be_chosen_randomly",
        ),
    SInt8("collision_permutation_index"),
    SInt8("physics_permutation_index"),
    SInt8("unknown", VISIBLE=False),
    ENDIAN=">", SIZE=8
    )


hlmt_collision_region = Struct("collision_region",
    h3_string_id("name"),
    SInt8("collision_region_index"),
    SInt8("physics_region_index"),
    SInt8("unknown_0", VISIBLE=False),
    SInt8("unknown_1", VISIBLE=False),
    h3_reflexive("permutations", hlmt_collision_region_permutation),
    ENDIAN=">", SIZE=20
    )


hlmt_node = Struct("node",
    h3_string_id("name"),
    SInt16("parent_node"),
    SInt16("first_child_node"),
    SInt16("next_sibling_node"),
    SInt16("import_node_index"),
    QStruct("default_translation", INCLUDE=xyz_float),
    QStruct("default_rotation", INCLUDE=ijkw_float),
    Float("default_scale"),
    QStruct("inverse_forward", INCLUDE=ijk_float),
    QStruct("inverse_left", INCLUDE=ijk_float),
    QStruct("inverse_up", INCLUDE=ijk_float),
    QStruct("inverse_position", INCLUDE=xyz_float),
    ENDIAN=">", SIZE=92
    )


hlmt_model_object_data = Struct("model_object_data",
    SEnum16("type", *hlmt_model_object_data_type),
    SInt16("unknown", VISIBLE=False),
    QStruct("offset", INCLUDE=xyz_float),
    Float("radius"),
    ENDIAN=">", SIZE=20
    )


hlmt_unknown_3 = Struct("unknown_3",
    h3_string_id("region"),
    h3_string_id("permutation"),
    VISIBLE=False,
    ENDIAN=">", SIZE=8
    )


hlmt_unknown_4 = Struct("unknown_4",
    h3_string_id("unknown_0", VISIBLE=False),
    BytesRaw("unknown_1", SIZE=4, VISIBLE=False),
    VISIBLE=False,
    ENDIAN=">", SIZE=8
    )


hlmt_unknown_5 = Struct("unknown_5",
    h3_string_id("marker_0", VISIBLE=False),
    BytesRaw("unknown_0", SIZE=4, VISIBLE=False),
    h3_string_id("marker_1", VISIBLE=False),
    BytesRaw("unknown_1", SIZE=8, VISIBLE=False),
    VISIBLE=False,
    ENDIAN=">", SIZE=20
    )


hlmt_body = Struct("tagdata",
    h3_dependency("model"),
    h3_dependency("collision_model"),
    h3_dependency("animation"),
    h3_dependency("physics_model"),
    Float("reduce_to_l1_super_low"),
    Float("reduce_to_l2_low"),
    Float("reduce_to_l3_medium"),
    Float("reduce_to_l4_high"),
    Float("reduce_to_l5_super_high"),
    h3_dependency("lod_model"),
    h3_reflexive("variants", hlmt_variant),
    h3_reflexive("instance_groups", hlmt_instance_group),
    h3_reflexive("materials", hlmt_material),
    h3_reflexive("new_damage_info", hlmt_new_damage_info),
    h3_reflexive("targets", hlmt_target),
    h3_reflexive("collision_regions", hlmt_collision_region),
    h3_reflexive("nodes", hlmt_node),
    BytesRaw("unknown_0", SIZE=4, VISIBLE=False),
    h3_reflexive("model_object_data", hlmt_model_object_data),
    h3_dependency("primary_dialog"),
    h3_dependency("secondary_dialog"),
    Bool32("flags",
        "active_camo_always_on",
        "active_camo_always_merge",
        "active_camo_never_merge",
        ),
    h3_string_id("default_dialogue_effect"),
    Bool32("render_only_node_flags_1", *("node_%s" % i for i in range(32)), VISIBLE=False),
    Bool32("render_only_node_flags_2", *("node_%s" % i for i in range(32, 64)), VISIBLE=False),
    Bool32("render_only_node_flags_3", *("node_%s" % i for i in range(64, 96)), VISIBLE=False),
    Bool32("render_only_node_flags_4", *("node_%s" % i for i in range(96, 128)), VISIBLE=False),
    Bool32("render_only_node_flags_5", *("node_%s" % i for i in range(128, 160)), VISIBLE=False),
    Bool32("render_only_node_flags_6", *("node_%s" % i for i in range(160, 192)), VISIBLE=False),
    Bool32("render_only_node_flags_7", *("node_%s" % i for i in range(192, 224)), VISIBLE=False),
    Bool32("render_only_node_flags_8", *("node_%s" % i for i in range(224, 256)), VISIBLE=False),
    Bool32("render_only_section_flags_1", *("section_%s" % i for i in range(32)), VISIBLE=False),
    Bool32("render_only_section_flags_2", *("section_%s" % i for i in range(32, 64)), VISIBLE=False),
    Bool32("render_only_section_flags_3", *("section_%s" % i for i in range(64, 96)), VISIBLE=False),
    Bool32("render_only_section_flags_4", *("section_%s" % i for i in range(96, 128)), VISIBLE=False),
    Bool32("render_only_section_flags_5", *("section_%s" % i for i in range(128, 160)), VISIBLE=False),
    Bool32("render_only_section_flags_6", *("section_%s" % i for i in range(160, 192)), VISIBLE=False),
    Bool32("render_only_section_flags_7", *("section_%s" % i for i in range(192, 224)), VISIBLE=False),
    Bool32("render_only_section_flags_8", *("section_%s" % i for i in range(224, 256)), VISIBLE=False),
    Bool32("runtime_flags",
        "contains_runtime_nodes",
        VISIBLE=False,
        ),
    BytesRaw("scenario_load_parameters_block", SIZE=12, VISIBLE=False),
    SInt16("unknown_1", VISIBLE=False),
    SInt16("unknown_2", VISIBLE=False),
    h3_reflexive("unknown_3", hlmt_unknown_3),
    h3_reflexive("unknown_4", hlmt_unknown_4),
    h3_reflexive("unknown_5", hlmt_unknown_5),
    h3_dependency("shield_impact_third_person"),
    h3_dependency("shield_impact_first_person"),
    ENDIAN=">", SIZE=392
    )


def get():
    return hlmt_def

hlmt_def = TagDef("hlmt",
    h3_blam_header('hlmt'),
    hlmt_body,

    ext=".%s" % h3_tag_class_fcc_to_ext["hlmt"], endian=">", tag_cls=H3Tag
    )



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

from reclaimer.h2.constants import HALO2_MAP_TYPES


def split_raw_pointer(ptr):
    return ptr & 0x3FffFFff, HALO2_MAP_TYPES[(ptr>>30)&3]



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

from math import pi

from .shdr import *
from .objs.shdr import ShdrTag
from supyr_struct.defs.tag_def import TagDef

ripple = Struct("ripple",
    Pad(4),
    float_zero_to_one("contribution_factor"),
    Pad(32),
    Float("animation_angle",
        MIN=0.0, MAX=2*pi, UNIT_SCALE=180/pi, SIDETIP="[0,360]"),  # radians
    Float("animation_velocity", UNIT_SCALE=per_sec_unit_scale),
    Struct("map_offset", INCLUDE=ij_float),
    UInt16("map_repeats"),
    UInt16("map_index"),
    SIZE=76
    )


swat_attrs = Struct("swat_attrs",
    #Water Shader Properties
    Struct("water_shader",
        Bool16("flags",
            "base_map_alpha_modulates_reflection",
            "base_map_color_modulates_background",
            "atmospheric_fog",
            "draw_before_fog",
            ),
        Pad(34),
        dependency("base_map", "bitm"),
        Pad(16),
        float_zero_to_one("perpendicular_brightness"),
        Struct("perpendicular_tint_color", INCLUDE=rgb_float),
        float_zero_to_one("parallel_brightness"),
        Struct("parallel_tint_color", INCLUDE=rgb_float),
        Pad(16),
        dependency("reflection_map", "bitm"),

        Pad(16),
        Float("ripple_animation_angle",
            MIN=0.0, MAX=2*pi, UNIT_SCALE=180/pi, SIDETIP="[0,360]"),  # radians
        Float("ripple_animation_velocity", UNIT_SCALE=per_sec_unit_scale),
        Float("ripple_scale"),
        dependency("ripple_maps", "bitm"),
        UInt16("ripple_mipmap_levels"),
        Pad(2),
        float_zero_to_one("ripple_mipmap_fade_factor"),
        Float("ripple_mipmap_detail_bias"),
        ),

    Pad(64),
    reflexive("ripples", ripple, 4),
    SIZE=280
    )

swat_body = Struct("tagdata",
    shdr_attrs,
    swat_attrs,
    SIZE=320,
    )

def get():
    return swat_def

swat_def = TagDef("swat",
    blam_header('swat', 2),
    swat_body,

    ext=".shader_transparent_water", endian=">", tag_cls=ShdrTag
    )



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

from reclaimer.h2.common_descs import *
from reclaimer.h2.defs.objs.tag import H2Tag
from supyr_struct.defs.tag_def import TagDef

# This is the reflexive that needs the MMAP struct headering the tbfd inside
rawdata_reflexive = h2_reflexive("data",
    SInt8("byte", COMMENT=(
        "\n            DONT TOUCH THIS!\n\n" +
        "It is actually rawdata treated as a reflexive.\n")),
    1024, EDITABLE=False
    )

runtime_property = Struct("runtime property",
    h2_dependency("diffuse map", "bitm"),
    Struct("lightmap",
        h2_dependency("emissive map", "bitm"),
        QStruct("emissive color", INCLUDE=rgb_float),
        Float("emissive power"),
        Float("resolution scale"),
        Float("half life"),
        Float("diffuse scale"),
        ),
    h2_dependency("alpha test map", "bitm"),
    h2_dependency("translucent map", "bitm"),
    Struct("more lightmap",
        QStruct("transparent color", INCLUDE=rgb_float),
        Float("transparent alpha"),
        Float("foliage scale")
        ),
    SIZE=80
    )

animation_property = Struct("animation property",
    UEnum16("type",
        "bitmap scale uniform",
        "bitmap scale x",
        "bitmap scale y",
        "bitmap scale z",
        "bitmap translation x",
        "bitmap translation y",
        "bitmap translation z",
        "bitmap rotation angle",
        "bitmap rotation axis x",
        "bitmap rotation axis y",
        "bitmap rotation axis z",
        "value",
        "color",
        "bitmap index",
        ),
    Pad(2),
    h2_string_id("input name"),
    h2_string_id("range name"),
    float_sec("time period"),
    rawdata_reflexive,
    SIZE=24
    )

global_parameters = Struct("global parameters",
    h2_string_id("material name"),
    UEnum16("type",
        "bitmap",
        "value",
        "color",
        "switch",
        ),
    Pad(2),
    h2_dependency("bitmap", "bitm"),
    Float("constant value"),
    QStruct("constant color", INCLUDE=rgb_float),
    h2_reflexive("animation properties", animation_property, 1024),
    SIZE=40
    )

postprocess_bitmap = QStruct("postprocess bitmap",
    UInt32("bitmap reference"),
    UInt32("bitmap index"),
    Float("log bitmap dimension"),
    SIZE=12, EDITABLE=False
    )

pixel_constants = QStruct("pixel constants", INCLUDE=argb_float, SIZE=16)

vertex_constants = QStruct("vertex constants",
    INCLUDE=ijkw_float, SIZE=16, EDITABLE=False)

level_of_detail = Struct("level of detail",
    Bool32("available layers", *["layer %s" % i for i in range(32)]),
    UInt16("block index data"),
    SIZE=6, EDITABLE=False
    )

block_index = UInt16("block index", EDITABLE=False)

implementation = QStruct("implementation",
    UInt16("block index data1"),
    UInt16("block index data2"),
    UInt16("block index data3"),
    UInt16("block index data4"),
    UInt16("block index data5"),
    SIZE=10, EDITABLE=False
    )

overlay = Struct("overlay",
    h2_string_id("input name"),
    h2_string_id("range name"),
    float_sec("time period"),
    rawdata_reflexive,
    SIZE=20
    )

overlay_reference = QStruct("overlay reference",
    UInt16("overlay index"),
    UInt16("transform index"),
    SIZE=4, EDITABLE=False
    )

animated_parameter_reference = QStruct("animated parameter reference",
    Pad(3),
    UInt8("parameter index"),
    SIZE=4, EDITABLE=False
    )

bitmap_property = QStruct("bitmap property",
    UInt16("bitmap index"),
    UInt16("animated parameter index"),
    SIZE=4, EDITABLE=False
    )

color_property = QStruct("color property",
    INCLUDE=rgb_float, EDITABLE=False)

shader_pass = Struct("pass",
    h2_dependency("template", "spas"),
    block_index,
    SIZE=10
    )

old_implementation = QStruct("implementation",
    *[UInt16("block index data%s" % (i+1)) for i in range(22)],
    SIZE=44, EDITABLE=False
    )

bitmap_index = UInt8("bitmap index", EDITABLE=False)


# ALL OF THE BLOCKS BELOW HERE DONT SEEM RIGHT.
# You don't just put a data in a spot that is off alignment.
bitmap_transform = Struct("bitmap transform",
    UInt8("parameter index"),
    UInt8("bitmap transform index"),
    Float("value"),
    SIZE=6, EDITABLE=False
    )

value = QStruct("value",
    UInt8("parameter index"),
    Float("value"),
    SIZE=5, EDITABLE=False
    )

color = Struct("color",
    UInt8("parameter index", EDITABLE=False),
    QStruct("color", INCLUDE=rgb_float),
    SIZE=13
    )

bitmap_transform_overlay = Struct("bitmap transform overlay",
    UInt8("parameter index", EDITABLE=False),
    UInt8("transform index", EDITABLE=False),
    UInt8("animation property type", EDITABLE=False),
    h2_string_id("input name"),
    h2_string_id("range name"),
    float_sec("time period"),
    rawdata_reflexive,
    SIZE=23
    )

value_overlay = Struct("value overlay",
    UInt8("parameter index", EDITABLE=False),
    h2_string_id("input name"),
    h2_string_id("range name"),
    float_sec("time period"),
    rawdata_reflexive,
    SIZE=21
    )

color_overlay = Struct("color overlay", INCLUDE=value_overlay)

spp_vertex_shader_constant = QStruct("vertex shader constant",
    UInt8("register index"),
    UInt8("register bank"),
    Float("data1"),
    Float("data2"),
    Float("data3"),
    Float("data4"),
    SIZE=18, EDITABLE=False
    )

render_state = QStruct("texture stage state",
    UInt8("state index"),
    UInt32("state value"),
    SIZE=5, EDITABLE=False
    )

texture_stage_state = QStruct("texture stage state",
    UInt8("state index"),
    UInt8("stage index"),
    UInt32("state value"),
    SIZE=6, EDITABLE=False
    )

# ALL OF THE BLOCKS ABOVE HERE DONT SEEM RIGHT.
# You don't just put a data in a spot that is off alignment.

render_state_parameter = QStruct("render state parameter",
    UInt8("parameter index"),
    UInt8("parameter type"),
    UInt8("state index"),
    SIZE=3, EDITABLE=False
    )

texture_stage_state_parameter = QStruct("texture stage state parameter",
    UInt8("parameter index"),
    UInt8("parameter type"),
    UInt8("state index"),
    UInt8("stage index"),
    SIZE=4, EDITABLE=False
    )

texture = QStruct("texture",
    UInt8("stage index"),
    UInt8("parameter index"),
    UInt8("global texture index"),
    UInt8("flags"),
    SIZE=4, EDITABLE=False
    )

vertex_shader_constant = QStruct("vertex shader constant",
    UInt8("register index"),
    UInt8("parameter index"),
    UInt8("destination mask"),
    UInt8("scale by texture stage"),
    SIZE=4, EDITABLE=False
    )

old_level_of_detail = Struct("old level of detail",
    Float("projected height percentage"),
    Bool32("available layers",
        *["layer%s" % i for i in range(25)], EDITABLE=False),
    Struct("postprocess",
        h2_reflexive("layers", block_index, 25),
        h2_reflexive("passes", shader_pass, 1024),
        h2_reflexive("implementations", old_implementation, 1024),
        h2_reflexive("bitmaps", bitmap_index, 1024),
        h2_reflexive("bitmap transforms", bitmap_transform, 1024),
        h2_reflexive("values", value, 1024),
        h2_reflexive("colors", color, 1024),
        h2_reflexive("bitmap transform overlays",
            bitmap_transform_overlay, 1024),
        h2_reflexive("value overlays", value_overlay, 1024),
        h2_reflexive("color overlays", color_overlay, 1024),
        h2_reflexive("vertex shader constants",
            spp_vertex_shader_constant, 1024),
        ),
    h2_reflexive("render states", render_state, 1024),
    h2_reflexive("texture stage states", texture_stage_state, 1024),
    h2_reflexive("render state parameters", render_state_parameter, 1024),
    h2_reflexive("texture stage state parameters",
        texture_stage_state_parameter, 1024),
    h2_reflexive("textures", texture, 1024),
    h2_reflexive("vn constants", vertex_shader_constant, 1024),
    h2_reflexive("cn constants", vertex_shader_constant, 1024),
    SIZE=152
    )

postprocess_definition = Struct("postprocess definitions",
    UInt32("shader template index", EDITABLE=False),
    h2_reflexive("bitmaps",          global_parameters,   1024),
    h2_reflexive("pixel constants",  pixel_constants,     1024),
    h2_reflexive("vertex constants", vertex_constants,    1024),
    h2_reflexive("levels of detail", level_of_detail,     1024),
    h2_reflexive("layers",           block_index,         1024),
    h2_reflexive("passes",           block_index,         1024),
    h2_reflexive("implementations",  implementation,      1024),
    h2_reflexive("overlays",         overlay,             1024),
    h2_reflexive("overlay references", overlay_reference, 1024),
    h2_reflexive("animated parameters", block_index,      1024),
    h2_reflexive("animated parameter references",
        animated_parameter_reference, 1024
        ),
    h2_reflexive("bitmap properties", bitmap_property, 5),
    h2_reflexive("color properties",  color_property,  2),
    h2_reflexive("value properties",  Float("value"),  6),
    h2_reflexive("old levels of detail", old_level_of_detail,  1024),
    SIZE=124
    )

predicted_resource = Struct("predicted resource",
    UEnum16("type",
        "bitmap",
        "sound",
        "render model geometry",
        "cluster geometry",
        "cluster instanced geometry",
        "lightmap geometry object buckets",
        "lightmap geometry instance buckets",
        "lightmap cluster bitmaps",
        "lightmap instance bitmaps"
        ),
    UInt16("resource index"),
    UInt32("predicted resource tag"),
    SIZE=8, EDITABLE=False
    )

shad_body = Struct("tagdata",
    h2_dependency("template", "stem"),
    h2_string_id("material name"),
    h2_reflexive("runtime properties", runtime_property, 1),
    Pad(2),
    Bool16("flags",
        "water",
        "sort first",
        "no active camo",
        ),
    h2_reflexive("global parameters", global_parameters, 64),
    h2_reflexive("postprocess definition", postprocess_definition, 1),
    Pad(4),
    h2_reflexive("predicted resources", predicted_resource, 2048),
    h2_dependency("light response", "slit"),
    UEnum16("shader lod bias",
        "none",
        {NAME: "times_4", GUI_NAME: "4x size"},
        {NAME: "times_2", GUI_NAME: "2x size"},
        {NAME: "times_half", GUI_NAME: "1/2x size"},
        {NAME: "times_quarter", GUI_NAME: "1/4x size"},
        "never",
        "cinematic"
        ),
    UEnum16("specular type",
        "none",
        "default",
        "dull",
        "shiny",
        ),
    UEnum16("lightmap type",
        "diffuse",
        "default specular",
        "dull specular",
        "shiny specular",
        ),
    Pad(2),
    Float("lightmap specular brightness"),
    float_neg_one_to_one("lightmap ambient bias"),
    h2_reflexive("postprocess properties", UInt32("bitmap group index"), 5),
    Float("added depth bias offset"),
    Float("added depth bias slope scale"),
    ENDIAN="<", SIZE=84
    )


def get():
    return shad_def

shad_def = TagDef("shad",
    h2_blam_header('shad'),
    shad_body,

    ext=".%s" % h2_tag_class_fcc_to_ext["shad"], endian="<"
    )



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

from ...common_descs import *
from .objs.tag import HekTag
from supyr_struct.defs.tag_def import TagDef

ui_widget_def = Struct("ui_widget_definition",
    dependency("ui_widget_definition", 'DeLa'),
    SIZE=16
    )

soul_body = Struct("tagdata",
    reflexive("ui_widget_definitions", ui_widget_def, 32,
        DYN_NAME_PATH='.ui_widget_definition.filepath'),
    SIZE=12,
    )


def get():
    return Soul_def

Soul_def = TagDef("Soul",
    blam_header('Soul'),
    soul_body,#lol Megaman X4

    ext=".ui_widget_collection", endian=">", tag_cls=HekTag
    )



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

############# Credits and version info #############
# Definition generated from Assembly XML tag def
#	 Date generated: 2018/12/03  04:56
#
# revision: 1		author: Assembly
# 	Generated plugin from scratch.
# revision: 2		author: Moses_of_Egypt
# 	Cleaned up and converted to SuPyr definition
#
####################################################

from ..common_descs import *
from .objs.tag import *
from supyr_struct.defs.tag_def import TagDef


cine_scene_unknown = Struct("scene_unknown",
    SInt32("unknown"),
    ENDIAN=">", SIZE=4
    )


cine_cinematic_scene = Struct("cinematic_scene",
    h3_dependency("scene"),
    ENDIAN=">", SIZE=16
    )


cine_body = Struct("tagdata",
    SInt32("unknown_0"),
    h3_reflexive("scene_unknown", cine_scene_unknown),
    h3_dependency("import_scenario"),
    SInt32("unknown_1"),
    h3_string_id("unknown_2"),
    SInt16("unknown_3"),
    SInt16("unknown_4"),
    SInt32("unknown_5"),
    Pad(20),
    SInt32("unknown_7"),
    Pad(12),
    SInt32("unknown_9"),
    h3_dependency("unknown_10"),
    h3_rawdata_ref("import_script_0"),
    h3_reflexive("cinematic_scenes", cine_cinematic_scene),
    h3_rawdata_ref("import_script_1"),
    h3_rawdata_ref("import_script_2"),
    ENDIAN=">", SIZE=176
    )


def get():
    return cine_def

cine_def = TagDef("cine",
    h3_blam_header('cine'),
    cine_body,

    ext=".%s" % h3_tag_class_fcc_to_ext["cine"], endian=">", tag_cls=H3Tag
    )



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

############# Credits and version info #############
# Definition generated from Assembly XML tag def
#	 Date generated: 2018/12/03  04:56
#
# revision: 1		author: Assembly
# 	Generated plugin from scratch.
# revision: 2		author: Lord Zedd
# 	H2 Port
# revision: 3		author: Moses_of_Egypt
# 	Cleaned up and converted to SuPyr definition
#
####################################################

from ..common_descs import *
from .objs.tag import *
from supyr_struct.defs.tag_def import TagDef


pphy_body = Struct("tagdata",
    Bool32("flags", *unknown_flags_32),
    Float("unknown_0", VISIBLE=False),
    Float("unknown_1", VISIBLE=False),
    Float("unknown_2", VISIBLE=False),
    BytesRaw("unknown_3", SIZE=16, VISIBLE=False),
    Float("density", VISIBLE=False),
    Float("air_friction"),
    Float("water_friction"),
    Float("surface_friction"),
    Float("elasticity"),
    BytesRaw("unknown_4", SIZE=12, VISIBLE=False),
    ENDIAN=">", SIZE=64
    )


def get():
    return pphy_def

pphy_def = TagDef("pphy",
    h3_blam_header('pphy'),
    pphy_body,

    ext=".%s" % h3_tag_class_fcc_to_ext["pphy"], endian=">", tag_cls=H3Tag
    )



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

from ...common_descs import *
from .objs.tag import HekTag
from supyr_struct.defs.tag_def import TagDef


scenario_description = Struct("scenario_description",
    dependency("descriptive_bitmap", "bitm"),
    dependency("displayed_map_name", "ustr"),
    ascii_str32("scenario_tag_directory_path"),
    SIZE=68
    )

mply_body = Struct("tagdata",
    reflexive("multiplayer_scenario_descriptions",
        scenario_description, 32, DYN_NAME_PATH='.scenario_tag_directory_path'),
    SIZE=12,
    )


def get():
    return mply_def

mply_def = TagDef("mply",
    blam_header('mply'),
    mply_body,

    ext=".multiplayer_scenario_description", endian=">", tag_cls=HekTag
    )



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

import zlib

from reclaimer.h2.common_descs import *
from supyr_struct.defs.tag_def import TagDef


def tag_name_table_name_pointer(parent=None, new_value=None, **kwargs):
    header = parent.parent.parent.parent
    if new_value is None:
        if parent.offset == -1:
            return header.tag_name_table_offset
        return header.tag_name_table_offset + parent.offset
    parent.offset = new_value - header.tag_name_table_offset


def string_id_table_name_pointer(parent=None, new_value=None, **kwargs):
    header = parent.parent.parent.parent
    if new_value is None:
        if parent.offset == -1:
            return header.string_id_table_offset
        return header.string_id_table_offset + parent.offset
    parent.offset = new_value - header.string_id_table_offset


tag_name_table_entry = Container("tag name table entry",
    SInt32("offset"),
    STEPTREE=CStrLatin1("tag_name", POINTER=tag_name_table_name_pointer),
    )

string_id_table_entry = Container("string id table entry",
    SInt32("offset"),
    STEPTREE=CStrLatin1("string", POINTER=string_id_table_name_pointer),
    )


tag_name_table = Array("tag name table",
    SIZE="..tag_name_count", SUB_STRUCT=tag_name_table_entry,
    POINTER="..tag_name_index_offset"
    )

string_id_table = Array("string id table",
    SIZE="..string_id_count", SUB_STRUCT=string_id_table_entry,
    POINTER="..string_id_index_offset"
    )


h2x_map_header = Struct("map header",
    UEnum32('head', ('head', 'head'), EDITABLE=False, DEFAULT='head'),
    UEnum32("version",
        ("halo1xbox",   5),
        ("halo1pcdemo", 6),
        ("halo1pc", 7),
        ("halo2", 8),
        ("halo3beta", 9),
        ("halo3", 11),
        ("halo1ce", 609),
        ),
    UInt32("decomp len"),
    UInt32("unknown0"),
    UInt32("tag index header offset"),
    UInt32("tag index data size"),
    UInt32("tag data size"),
    UInt32("tag data offset mask"),
    Pad(256),

    ascii_str32("build date"),
    UEnum16("map type",
        "sp",
        "mp",
        "ui",
        "shared",
        "sharedsp",
        ),
    Pad(2),
    UInt32("crc32"),
    Pad(16),
    UInt32("crazy data offset"),
    UInt32("crazy data len"),
    UInt32("string id padded offset"),  # what is this?
    UInt32("string id count"),
    UInt32("string id table size"),
    UInt32("string id index offset"),
    UInt32("string id table offset"),
    Pad(36),
    ascii_str32("map name"),
    Pad(4),
    StrLatin1("scenario tag path", SIZE=260),
    UInt32("tag name count"),
    UInt32("tag name table offset"),
    UInt32("tag name table size"),
    UInt32("tag name index offset"),
    UInt32("checksum1"),
    UInt32("unknown1"),
    UInt32("unknown2"),
    UInt32("raw table offset"),
    UInt32("raw table size"),
    UInt32("checksum2"),
    Pad(1300),
    UEnum32('foot', ('foot', 'foot'), EDITABLE=False, DEFAULT='foot'),
    SIZE=2048
    )

h2x_map_header_full = Struct("map header",
    INCLUDE=h2x_map_header,
    STEPTREE=Container("strings",
        string_id_table,
        tag_name_table,
        )
    )


h2v_map_header = Struct("map header",
    UEnum32('head', ('head', 'head'), EDITABLE=False, DEFAULT='head'),
    UEnum32("version",
        ("halo1xbox",   5),
        ("halo1pcdemo", 6),
        ("halo1pc", 7),
        ("halo2", 8),
        ("halo3", 11),
        ("halo1ce", 609),
        ),
    UInt32("decomp len"),
    UInt32("unknown0"),
    UInt32("tag index header offset"),
    UInt32("tag index data size"),
    UInt32("tag data size"),
    UInt32("tag data offset mask"),
    UInt32("virtual_address"),
    UInt32("dependency_map_offset"),  # -1 if not shared.map
    UInt32("dependency_map_size"),    #  0 if not shared.map
    Pad(256),

    ascii_str32("build date"),
    UEnum16("map type",
        "sp",
        "mp",
        "ui",
        "shared",
        "sharedsp",
        ),
    Pad(2),
    UInt32("crc32"),
    Pad(16),
    UInt32("crazy data offset"),
    UInt32("crazy data len"),
    UInt32("string id padded offset"),  # what is this?
    UInt32("string id count"),
    UInt32("string id table size"),
    UInt32("string id index offset"),
    UInt32("string id table offset"),
    Pad(36),
    ascii_str32("map name"),
    Pad(4),
    StrLatin1("scenario tag path", SIZE=260),
    UInt32("tag name count"),
    UInt32("tag name table offset"),
    UInt32("tag name table size"),
    UInt32("tag name index offset"),
    UInt32("checksum1"),
    UInt32("unknown4"),
    UInt32("unknown5"),
    UInt32("raw table offset"),
    UInt32("raw table size"),
    UInt32("checksum2"),
    Pad(1288),
    UEnum32('foot', ('foot', 'foot'), EDITABLE=False, DEFAULT='foot'),
    INCLUDE=h2x_map_header
    )

h2v_map_header_full = Struct("map header",
    INCLUDE=h2v_map_header,
    STEPTREE=Container("strings",
        string_id_table,
        tag_name_table,
        )
    )

h2_tag_type = Struct("tag header",
    UEnum32("class 1", GUI_NAME="primary tag class",   INCLUDE=valid_h2_tags),
    UEnum32("class 2", GUI_NAME="secondary tag class", INCLUDE=valid_h2_tags),
    UEnum32("class 3", GUI_NAME="tertiary tag class",  INCLUDE=valid_h2_tags)
    )

h2_tag_header = Struct("tag header",
    UEnum32("tag class", INCLUDE=valid_h2_tags),
    UInt32("id"),
    UInt32("offset"),
    UInt32("size"),
    SIZE=16
    )

h2_tag_types_array = Array("tag types",
    SIZE=".tag_types_count", SUB_STRUCT=h2_tag_type,
    )

h2_tag_index_array = TagIndex("tag index",
    SIZE=".tag_count", SUB_STRUCT=h2_tag_header,
    )

h2_tag_index = Container("tag index",
    UInt32("tag types offset"),
    UInt32("tag types count"),
    UInt32("tag index offset"),

    UInt32("scenario tag id"),
    UInt32("globals tag id"),

    UInt32("crc32"),
    UInt32("tag count"),
    UInt32("tag sig", EDITABLE=False, DEFAULT='tags'),

    h2_tag_types_array,
    h2_tag_index_array
    )

h2x_map_header_def = BlockDef(h2x_map_header)
h2x_map_header_full_def = BlockDef(h2x_map_header_full)
h2v_map_header_def = BlockDef(h2v_map_header)
h2v_map_header_full_def = BlockDef(h2v_map_header_full)
h2_tag_index_def = BlockDef(h2_tag_index)



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

'''
Names for the "physics" struct in the biped tag are courtesy of Sparky.
The source files where the information was taken from are here:

https://github.com/LiquidLightning/infernal/blob/master/infernal/inf_bipd.h

EDIT: updates physics struct with more accurate names determined by Kavawuvi
'''
from math import sqrt
from .objs.bipd import BipdTag
from .obje import *
from .unit import *

# replace the object_type enum one that uses
# the correct default value for this object
obje_attrs = dict(obje_attrs)
obje_attrs[0] = dict(obje_attrs[0], DEFAULT=0)

contact_point = Struct("contact_point",
    Pad(32),
    ascii_str32('marker_name'),
    SIZE=64
    )

bipd_attrs = Struct("bipd_attrs",
    Float("moving_turning_speed",
        SIDETIP="degrees/sec", UNIT_SCALE=180/pi),  # radians
    Bool32("flags",
        "turns_without_aiming",
        "uses_player_physics",
        "physics_pill_centered_at_origin",
        "spherical",
        "passes_through_other_bipeds",
        "can_climb_any_surface",
        "immune_to_falling_damage",
        "rotate_while_airborne",
        "uses_limp_body_physics",
        "has_no_dying_airborne",
        "random_speed_increase",
        "uses_old_player_physics",
        ),
    float_rad("stationary_turning_threshold"),  # radians

    Pad(16),
    SEnum16('A_in', *biped_inputs),
    SEnum16('B_in', *biped_inputs),
    SEnum16('C_in', *biped_inputs),
    SEnum16('D_in', *biped_inputs),
    dependency('DONT_USE', "jpt!"),

    QStruct("flying",
        float_rad("bank_angle"),  # radians
        float_sec("bank_apply_time", UNIT_SCALE=sec_unit_scale),  # seconds
        float_sec("bank_decay_time", UNIT_SCALE=sec_unit_scale),  # seconds
        Float("pitch_ratio"),
        float_wu_sec("max_velocity",
                     UNIT_SCALE=per_sec_unit_scale),  # world units/second
        float_wu_sec("max_sidestep_velocity",
                     UNIT_SCALE=per_sec_unit_scale),  # world units/second
        float_wu_sec_sq("acceleration",
                        UNIT_SCALE=per_sec_unit_scale),  # world units/second^2
        float_wu_sec_sq("deceleration",
                        UNIT_SCALE=per_sec_unit_scale),  # world units/second^2
        float_rad_sec("angular_velocity_maximum"),  # radians/second
        float_rad_sec_sq("angular_acceleration_maximum"),  # radians/second^2
        float_zero_to_one("crouch_velocity_modifier"),
        ),

    Pad(8),
    Struct("movement",
        float_rad("maximum_slope_angle"),  # radians
        float_rad("downhill_falloff_angle"),  # radians
        float_rad("downhill_cutoff_angle"),  # radians
        Float("downhill_velocity_scale"),
        float_rad("uphill_falloff_angle"),  # radians
        float_rad("uphill_cutoff_angle"),  # radians
        Float("uphill_velocity_scale"),

        Pad(24),
        dependency('footsteps', "foot"),
        ),

    Pad(24),
    QStruct("jumping_and_landing",
        float_wu_sec("jump_velocity", UNIT_SCALE=per_sec_unit_scale),
        Pad(28),
        float_sec("maximum_soft_landing_time", UNIT_SCALE=sec_unit_scale),
        float_sec("maximum_hard_landing_time", UNIT_SCALE=sec_unit_scale),
        float_wu_sec("minimum_soft_landing_velocity",
                     UNIT_SCALE=per_sec_unit_scale),  # world units/second
        float_wu_sec("minimum_hard_landing_velocity",
                     UNIT_SCALE=per_sec_unit_scale),  # world units/second
        float_wu_sec("maximum_hard_landing_velocity",
                     UNIT_SCALE=per_sec_unit_scale),  # world units/second
        float_wu_sec("death_hard_landing_velocity",
                     UNIT_SCALE=per_sec_unit_scale),  # world units/second
        ),

    Pad(20),
    QStruct("camera_collision_and_autoaim",
        float_wu("standing_camera_height"),
        float_wu("crouching_camera_height"),
        float_sec("crouch_transition_time", UNIT_SCALE=sec_unit_scale),

        Pad(24),
        float_wu("standing_collision_height"),
        float_wu("crouching_collision_height"),
        float_wu("collision_radius"),

        Pad(40),
        float_wu("autoaim_width"),
        ),

    Pad(108),
    QStruct("physics",
        # the default values below that aren't commented out are taken
        # from the cyborg.biped tag after saving it with guerilla.
        FlFloat("cosine_stationary_turning_threshold"),
        FlFloat("crouch_camera_velocity"),
        FlFloat("cosine_maximum_slope_angle",
            TOOLTIP=("negative is walking on walls.\n > 0.707107 is " +
                     "floating with contact points off the ground")),
        FlFloat("neg_sine_downhill_falloff_angle"),
        FlFloat("neg_sine_downhill_cutoff_angle"),
        FlFloat("sine_uphill_falloff_angle"),
        FlFloat("sine_uphill_cutoff_angle",
            TOOLTIP="does the same thing as the fp accel modifier?"),
        FlSInt16("root_node_index", DEFAULT=-1),
        FlSInt16("head_node_index", DEFAULT=-1),
        VISIBLE=False, SIZE=32
        ),

    reflexive("contact_points", contact_point, 2,
        DYN_NAME_PATH='.marker_name'),

    SIZE=516
    )

bipd_body = Struct("tagdata",
    obje_attrs,
    unit_attrs,
    bipd_attrs,
    SIZE=1268,
    )


def get():
    return bipd_def

bipd_def = TagDef("bipd",
    blam_header('bipd', 3),
    bipd_body,

    ext=".biped", endian=">", tag_cls=BipdTag
    )



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

from ...common_descs import *
from .objs.tag import HekTag
from supyr_struct.defs.tag_def import TagDef


color = Struct("color",
    ascii_str32('name'),
    QStruct("color", INCLUDE=argb_float),
    SIZE=48,
    )

colo_body = Struct("tagdata",
    reflexive("colors", color, 512, DYN_NAME_PATH='.name'),
    SIZE=12
    )

def get():
    return colo_def

colo_def = TagDef("colo",
    blam_header('colo'),
    colo_body,

    ext=".color_table", endian=">", tag_cls=HekTag
    )



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

# DO NOT REMOVE! __all__ is used to locate frozen definitions
__all__ = (
    "actr", "actv", "ant_", "antr", "bipd", "bitm", "boom", "cdmg", "coll",
    "colo", "cont", "ctrl", "deca", "DeLa", "devc", "devi", "dobc", "effe",
    "elec", "eqip", "flag", "fog_", "font", "foot", "garb", "glw_", "grhi",
    "hmt_", "hud_", "hudg", "item", "itmc", "jpt_", "lens", "lifi", "ligh",
    "mach", "matg", "metr", "mgs2", "mod2", "mode", "mply", "ngpr", "obje",
    "part", "pctl", "phys", "plac", "pphy", "proj", "rain", "sbsp", "scen",
    "scex", "schi", "scnr", "senv", "sgla", "shdr", "sky_", "smet", "snd_",
    "snde", "lsnd", "soso", "sotr", "Soul", "spla", "ssce", "str_", "swat",
    "tagc", "trak", "udlg", "unhi", "unit", "ustr", "vcky", "vehi", "weap",
    "wind", "wphi",
    )



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

############# Credits and version info #############
# Definition generated from Assembly XML tag def
#	 Date generated: 2018/12/03  04:56
#
# revision: 1		author: Assembly
# 	Generated plugin from scratch.
# revision: 2		author: DeadCanadian
# 	put in flags, bounding info, named tags, attachments, seats, cameras
# revision: 3		author: Lord Zedd
# 	Portin'
# revision: 4		author: Moses_of_Egypt
# 	Cleaned up and converted to SuPyr definition
#
####################################################

from ..common_descs import *
from .objs.tag import *
from supyr_struct.defs.tag_def import TagDef


gint_early_mover_propertie = Struct("early_mover_propertie",
    h3_string_id("name"),
    Pad(36),
    ENDIAN=">", SIZE=40
    )


gint_ai_propertie = Struct("ai_propertie",
    Bool32("flags",
        "destroyable_cover",
        "pathfinding_ignore_when_dead",
        "dynamic_cover",
        ),
    h3_string_id("ai_type_name"),
    BytesRaw("unknown", SIZE=4, VISIBLE=False),
    SEnum16("size", *bloc_ai_propertie_size),
    SEnum16("leap_jump_speed", *bloc_ai_propertie_leap_jump_speed),
    ENDIAN=">", SIZE=16
    )


gint_function = Struct("function",
    Bool32("flags",
        "invert",
        "mapping_does_not_controls_active",
        "always_active",
        "random_time_offset",
        ),
    h3_string_id("import_name"),
    h3_string_id("export_name"),
    h3_string_id("turn_off_with"),
    Float("minimum_value"),
    h3_rawdata_ref("default_function"),
    h3_string_id("scale_by"),
    ENDIAN=">", SIZE=44
    )


gint_attachment = Struct("attachment",
    h3_dependency("attachment"),
    h3_string_id("marker"),
    SEnum16("change_color", *bloc_attachment_change_color),
    SInt16("unknown", VISIBLE=False),
    h3_string_id("primary_scale"),
    h3_string_id("secondary_scale"),
    ENDIAN=">", SIZE=32
    )


gint_widget = Struct("widget",
    h3_dependency("type"),
    ENDIAN=">", SIZE=16
    )


gint_change_color_initial_permutation = Struct("initial_permutation",
    Pad(4),
    color_rgb_float("color_lower_bound"),
    color_rgb_float("color_upper_bound"),
    h3_string_id("variant_name"),
    ENDIAN=">", SIZE=32
    )


gint_change_color_function = Struct("function",
    Bool32("scale_flags",
        "blend_in_hsv",
        "more_colors",
        ),
    color_rgb_float("color_lower_bound"),
    color_rgb_float("color_upper_bound"),
    h3_string_id("darken_by"),
    h3_string_id("scale_by"),
    ENDIAN=">", SIZE=32
    )


gint_change_color = Struct("change_color",
    h3_reflexive("initial_permutations", gint_change_color_initial_permutation),
    h3_reflexive("functions", gint_change_color_function),
    ENDIAN=">", SIZE=24
    )


gint_predicted_resource = Struct("predicted_resource",
    SInt16("type"),
    SInt16("resource_index"),
    dependency_uint32("tag_index", VISIBLE=False),
    VISIBLE=False,
    ENDIAN=">", SIZE=8
    )


gint_multiplayer_object_propertie = Struct("multiplayer_object_propertie",
    Bool16("engine_flags",
        "capture_the_flag",
        "slayer",
        "oddball",
        "king_of_the_hill",
        "juggernaut",
        "territories",
        "assault",
        "vip",
        "infection",
        ),
    SEnum8("object_type", *bloc_multiplayer_object_propertie_object_type),
    Bool8("teleporter_flags",
        "disallows_players",
        "allows_land_vehicles",
        "allows_heavy_vehicles",
        "allows_flying_vehicles",
        "allows_projectiles",
        ),
    Bool16("flags",
        "editor_only",
        ),
    SEnum8("shape", *bloc_multiplayer_object_propertie_shape),
    SEnum8("spawn_timer_mode", *bloc_multiplayer_object_propertie_spawn_timer_mode),
    SInt16("spawn_time"),
    SInt16("abandon_time"),
    Float("radius_width"),
    Float("length"),
    Float("top"),
    Float("bottom"),
    Float("unknown_0", VISIBLE=False),
    Float("unknown_1", VISIBLE=False),
    Float("unknown_2", VISIBLE=False),
    SInt32("unknown_3", VISIBLE=False),
    SInt32("unknown_4", VISIBLE=False),
    h3_dependency("child_object"),
    SInt32("unknown_5"),
    h3_dependency("shape_shader"),
    h3_dependency("unknown_shader"),
    h3_dependency("unknown_6"),
    h3_dependency("unknown_7"),
    h3_dependency("unknown_8"),
    h3_dependency("unknown_9"),
    h3_dependency("unknown_10"),
    h3_dependency("unknown_11"),
    ENDIAN=">", SIZE=196
    )


gint_metagame_propertie = Struct("metagame_propertie",
    Bool8("flags",
        "must_have_active_seats",
        ),
    SEnum8("unit", *bloc_metagame_propertie_unit),
    SEnum8("classification", *bloc_metagame_propertie_classification),
    SInt8("unknown_0", VISIBLE=False),
    SInt16("points"),
    SInt16("unknown_1", VISIBLE=False),
    ENDIAN=">", SIZE=8
    )


gint_camera_track = Struct("camera_track",
    h3_dependency("track"),
    ENDIAN=">", SIZE=16
    )


gint_unknown_7 = Struct("unknown_7",
    Array("unknown_array", SUB_STRUCT=Float("unknown"), SIZE=19, VISIBLE=False),
    VISIBLE=False,
    ENDIAN=">", SIZE=76
    )


gint_posture = Struct("posture",
    h3_string_id("name"),
    QStruct("pill_offset", INCLUDE=ijk_float),
    ENDIAN=">", SIZE=16
    )


gint_hud_interface = Struct("hud_interface",
    h3_dependency("unit_hud_interface"),
    ENDIAN=">", SIZE=16
    )


gint_dialogue_variant = Struct("dialogue_variant",
    SInt16("variant_number"),
    SInt16("unknown", VISIBLE=False),
    h3_dependency("dialogue"),
    ENDIAN=">", SIZE=20
    )


gint_powered_seat = Struct("powered_seat",
    Float("driver_powerup_time"),
    Float("driver_powerdown_time"),
    ENDIAN=">", SIZE=8
    )


gint_weapon = Struct("weapon",
    h3_dependency("weapon"),
    ENDIAN=">", SIZE=16
    )


gint_seat_camera_track = Struct("camera_track",
    h3_dependency("track"),
    ENDIAN=">", SIZE=16
    )


gint_seat_unknown_6 = Struct("unknown_6",
    Array("unknown_array", SUB_STRUCT=Float("unknown"), SIZE=19, VISIBLE=False),
    VISIBLE=False,
    ENDIAN=">", SIZE=76
    )


gint_seat_unit_hud_interface = Struct("unit_hud_interface",
    h3_dependency("unit_hud_interface"),
    ENDIAN=">", SIZE=16
    )


gint_seat = Struct("seat",
    Bool32("flags",
        "invisible",
        "locked",
        "driver",
        "gunner",
        "third_person_camera",
        "allows_weapons",
        "third_person_on_enter",
        "first_person_camera_slaved_to_gun",
        "allow_vehicle_communication_animations",
        "not_valid_without_driver",
        "allow_ai_noncombatants",
        "boarding_seat",
        "ai_firing_disabled_by_max_acceleration",
        "boarding_enters_seat",
        "boarding_need_any_passenger",
        "controls_open_and_close",
        "invalid_for_player",
        "invalid_for_non_player",
        "gunner_player_only",
        "invisible_under_major_damage",
        ("detachable_weapon", 1 << 22),
        ),
    h3_string_id("seat_animation"),
    h3_string_id("seat_marker_name"),
    h3_string_id("entry_marker_s_name"),
    h3_string_id("boarding_grenade_marker"),
    h3_string_id("boarding_grenade_string"),
    h3_string_id("boarding_melee_string"),
    h3_string_id("detach_weapon_string"),
    Float("ping_scale"),
    Float("turnover_time"),
    QStruct("acceleration_range", INCLUDE=ijk_float),
    Float("acceleration_action_scale"),
    Float("acceleration_attach_scale"),
    Float("ai_scariness"),
    SEnum16("ai_seat_type", *gint_seat_ai_seat_type),
    SInt16("boarding_seat"),
    Float("listener_interpolation_factor"),
    QStruct("yaw_rate_bounds", INCLUDE=from_to),
    QStruct("pitch_rate_bounds", INCLUDE=from_to),
    Float("unknown_0", VISIBLE=False),
    QStruct("speed_reference", INCLUDE=from_to),
    Float("speed_exponent"),
    SInt16("unknown_1", VISIBLE=False),
    SInt16("unknown_2", VISIBLE=False),
    h3_string_id("camera_marker_name"),
    h3_string_id("camera_submerged_marker_name"),
    float_rad("pitch_auto_level"),
    from_to_rad("pitch_range"),
    h3_reflexive("camera_tracks", gint_seat_camera_track),
    float_rad("unknown_3", VISIBLE=False),
    float_rad("unknown_4", VISIBLE=False),
    float_rad("unknown_5", VISIBLE=False),
    h3_reflexive("unknown_6", gint_seat_unknown_6),
    h3_reflexive("unit_hud_interface", gint_seat_unit_hud_interface),
    h3_string_id("enter_seat_string"),
    from_to_rad("yaw_range"),
    h3_dependency("built_in_gunner"),
    Float("entry_radius"),
    float_rad("entry_marker_cone_angle"),
    float_rad("entry_marker_facing_angle"),
    Float("maximum_relative_velocity"),
    h3_string_id("invisible_seat_region"),
    SInt32("runtime_invisible_seat_region_index"),
    ENDIAN=">", SIZE=228
    )


gint_buckling_parameter = Struct("buckling_parameter",
    Float("unknown_0"),
    SInt32("unknown_1"),
    Float("unknown_2"),
    SInt32("unknown_3"),
    Array("unknown_array", SUB_STRUCT=Float("unknown"), SIZE=5),
    h3_string_id("marker"),
    Float("scan_distance_1_float"),
    Float("scan_distance_2_float"),
    SInt32("scan_distance_1_integer"),
    SInt32("scan_distance_2_integer"),
    float_rad("min_0"),
    float_rad("max_0"),
    float_rad("min_1"),
    float_rad("max_1"),
    h3_string_id("buckle_animation"),
    h3_string_id("buckle_wobble_animation_0"),
    h3_string_id("buckle_wobble_animation_1"),
    Float("unknown_4"),
    Float("unknown_5"),
    ENDIAN=">", SIZE=92
    )


gint_body = Struct("tagdata",
    SEnum16("object_type", *bloc_object_type),
    Bool16("flags_0",
        "does_not_cast_shadow",
        "search_cardinal_direction_lightmaps",
        ("not_a_pathfinding_obstacle", 1 << 3),
        "extension_of_parent",
        "does_not_cause_collision_damage",
        "early_mover",
        "early_mover_localized_physics",
        "use_static_massive_lightmap_sample",
        "object_scales_attachments",
        "inherits_player_s_appearance",
        "dead_bipeds_can_t_localize",
        "attach_to_clusters_by_dynamic_sphere",
        "effects_created_by_this_object_do_not_spawn_objects_in_multiplayer",
        ),
    Float("bounding_radius"),
    QStruct("bounding_offset", INCLUDE=xyz_float),
    Float("acceleration_scale"),
    SEnum16("lightmap_shadow_mode_size", *bloc_lightmap_shadow_mode_size),
    SEnum8("sweetener_size", *bloc_sweetener_size),
    SEnum8("water_density", *bloc_water_density),
    SInt32("unknown_0", VISIBLE=False),
    Float("dynamic_light_sphere_radius"),
    QStruct("dynamic_light_sphere_offset", INCLUDE=xyz_float),
    h3_string_id("default_model_variant"),
    h3_dependency("model"),
    h3_dependency("crate_object"),
    h3_dependency("collision_damage"),
    h3_reflexive("early_mover_properties", gint_early_mover_propertie),
    h3_dependency("creation_effect"),
    h3_dependency("material_effects"),
    h3_dependency("melee_impact"),
    h3_reflexive("ai_properties", gint_ai_propertie),
    h3_reflexive("functions", gint_function),
    SInt16("hud_text_message_index"),
    SInt16("unknown_1", VISIBLE=False),
    h3_reflexive("attachments", gint_attachment),
    h3_reflexive("widgets", gint_widget),
    h3_reflexive("change_colors", gint_change_color),
    h3_reflexive("predicted_resources", gint_predicted_resource),
    h3_reflexive("multiplayer_object_properties", gint_multiplayer_object_propertie),
    Bool32("flags_1",
        "circular_aiming",
        "destroyed_after_dying",
        "half_speed_interpolation",
        "fires_from_camera",
        "entrance_inside_bounding_sphere",
        "doesn_t_show_readied_weapon",
        "causes_passenger_dialogue",
        "resists_pings",
        "melee_attack_is_fatal",
        "dont_reface_during_pings",
        "has_no_aiming",
        "simple_creature",
        "impact_melee_attaches_to_unit",
        "impact_melee_dies_on_shield",
        "cannot_open_doors_automatically",
        "melee_attackers_cannot_attach",
        "not_instantly_killed_by_melee",
        "shield_sapping",
        "runs_around_flaming",
        "inconsequential",
        "special_cinematic_unit",
        "ignored_by_autoaiming",
        "shields_fry_infection_forms",
        "can_dual_wield",
        ("acts_as_gunner_for_parent", 1 << 25),
        "controlled_by_parent_gunner",
        "parent_s_primary_weapon",
        "unit_has_boost",
        ),
    SEnum16("default_team", *crea_default_team),
    SEnum16("constant_sound_volume", *gint_constant_sound_volume),
    h3_reflexive("metagame_properties", gint_metagame_propertie),
    h3_dependency("integrated_light_toggle"),
    float_rad("camera_field_of_view"),
    Float("camera_stiffness"),
    SInt16("unknown_2", VISIBLE=False),
    SInt16("unknown_3", VISIBLE=False),
    h3_string_id("camera_marker_name"),
    h3_string_id("camera_submerged_marker_name"),
    float_rad("pitch_auto_level"),
    from_to_rad("pitch_range"),
    h3_reflexive("camera_tracks", gint_camera_track),
    float_rad("unknown_4", VISIBLE=False),
    float_rad("unknown_5", VISIBLE=False),
    float_rad("unknown_6", VISIBLE=False),
    h3_reflexive("unknown_7", gint_unknown_7),
    QStruct("acceleration_range", INCLUDE=ijk_float),
    Float("acceleration_action_scale"),
    Float("acceleration_attach_scale"),
    Float("soft_ping_threshold"),
    Float("soft_ping_interrupt_time"),
    Float("hard_ping_threshold"),
    Float("hard_ping_interrupt_time"),
    Float("feign_death_threshold"),
    Float("feign_death_time"),
    Float("distance_of_evade_animation"),
    Float("distance_of_dive_animation"),
    Float("stunned_movement_threshold"),
    Float("feign_death_chance"),
    Float("feign_repeat_chance"),
    h3_dependency("spawned_turret_character"),
    QStruct("spawned_actor_count", INCLUDE=from_to_sint16),
    Float("spawned_velocity"),
    float_rad("aiming_velocity_maximum"),
    float_rad("aiming_acceleration_maximum"),
    Float("casual_aiming_modifier"),
    float_rad("looking_velocity_maximum"),
    float_rad("looking_acceleration_maximum"),
    h3_string_id("right_hand_node"),
    h3_string_id("left_hand_node"),
    h3_string_id("preferred_gun_node"),
    h3_dependency("melee_damage"),
    h3_dependency("boarding_melee_damage"),
    h3_dependency("boarding_melee_response"),
    h3_dependency("ejection_melee_damage"),
    h3_dependency("ejection_melee_response"),
    h3_dependency("landing_melee_damage"),
    h3_dependency("flurry_melee_damage"),
    h3_dependency("obstacle_smash_damage"),
    SEnum16("motion_sensor_blip_size", *crea_motion_sensor_blip_size),
    SEnum16("item_scale", *gint_item_scale),
    h3_reflexive("postures", gint_posture),
    h3_reflexive("hud_interfaces", gint_hud_interface),
    h3_reflexive("dialogue_variants", gint_dialogue_variant),
    Float("grenade_velocity"),
    SEnum16("grenade_type", *gint_grenade_type),
    SInt16("grenade_count"),
    h3_reflexive("powered_seats", gint_powered_seat),
    h3_reflexive("weapons", gint_weapon),
    h3_reflexive("seats", gint_seat),
    Float("emp_radius"),
    h3_dependency("emp_effect"),
    h3_dependency("boost_collision_damage"),
    Float("boost_peak_power"),
    Float("boost_rise_power"),
    Float("boost_peak_time"),
    Float("boost_fall_power"),
    Float("boost_dead_time"),
    Float("lipsync_attack_weight"),
    Float("lipsync_decay_weight"),
    h3_dependency("detach_damage"),
    h3_dependency("detached_weapon"),
    Pad(4),
    Float("unknown_9"),
    Float("unknown_10"),
    Float("animation_speed"),
    Float("vertical_force"),
    Float("leg_attach_radius"),
    h3_reflexive("buckling_parameters", gint_buckling_parameter),
    Float("toe_joint_extended_fraction"),
    ENDIAN=">", SIZE=820
    )


def get():
    return gint_def

gint_def = TagDef("gint",
    h3_blam_header('gint'),
    gint_body,

    ext=".%s" % h3_tag_class_fcc_to_ext["gint"], endian=">", tag_cls=H3Tag
    )



#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

from ...common_descs import *
from .objs.tag import HekTag
from supyr_struct.defs.tag_def import TagDef

message_element = Struct("message_element",
    UEnum8("type",
        "text",
        "icon",
        EDITABLE=False
        ),
    Union("data",
        CASE=".type.enum_name",
        CASES=dict(
            text=UEnum8("length",
                *({GUI_NAME: str(i), NAME: "_%s" % i} for i in range(256))),
            icon=UEnum8("icon_type",
                *({GUI_NAME: name, NAME: name} for name in hmt_icon_types)
                ),
            ),
        EDITABLE=False
        ),
    SIZE=2,
    )

message = Struct("message",
    ascii_str32("name"),
    SInt16("text_start", GUI_NAME="start index into text blob"),
    SInt16("element_index", GUI_NAME="start index of message block"),
    SInt8("element_count"),
    Computed("message_preview", WIDGET=HaloHudMessageTextFrame),
    SIZE=64
    )

hmt__body = Struct("tagdata",
    rawtext_ref("string", FlStrUTF16Data, max_size=65536,
        VISIBLE=False, EDITABLE=False),
    reflexive("message_elements", message_element, 8192),
    reflexive("messages", message, 1024, DYN_NAME_PATH='.name'),
    SIZE=128,
    )


def get():
    return hmt__def

hmt__def = TagDef("hmt ",
    blam_header('hmt '),
    hmt__body,

    ext=".hud_message_text", endian=">", tag_cls=HekTag
    )
